#!/usr/bin/perl
#
# 	CDR parser for ZTE log files
# 
#


use Time::HiRes;
use Socket;
use Sys::Hostname;
use Fcntl;
use Time::Local;

$|=1;

	printf STDERR "Main CDR processing loop started\n";

	while(<STDIN>) {
		$cdr_buf = $_;

		printf STDERR "Got ZTE string:\t$cdr_buf";

		if($cdr_buf !~ /^CHARGE/) {
			printf STDERR "Not a CHARGE record! Skipped!\n";
			next;
		}

		$cdr_buf =~ s/ /\!/;
		($ticket_head, $ticket_body) = split('!', $cdr_buf);
		
		printf STDERR "New ticket: $ticket_head\n";

		foreach $ticket (split('\$', $ticket_body)) {
			($record_type, $part_record_id, $caller_id_type, $caller_number, 
			 $called_id_type, $called_number, $start_time, $start_ticks,
			 $service_category, $end_time, $end_ticks, $release_reason,
		 	 $caller_type, $record_valid_id, $clock_id, $charge_flag, 
			 $attempt_call_id, $caller_analyze_id, $called_analyze_id,
			 $incoming_trunk_group, $outgoing_trunk_group, $supplementary_service,
			 $charge_party_id, $type_of_link_number, $link_number,$_,$bearer_services,
			 $terminal_services, $UUS1, $UUS3, $caller_special_number,
			 $called_special_number, $centrex_group_id, $type_of_billed_number, $billed_number,
			) = split('&', $ticket);

			printf STDERR "Record type: $record_type\n";
			printf STDERR "	Call start: $start_time\n";
			printf STDERR "	Call end time: $end_time\n";
			printf STDERR "	Calling from: $caller_number (trunk group: $incoming_trunk_group)\n";
			printf STDERR "	Called to: $called_number (trunk group: $outgoing_trunk_group)\n";

			# Put CDR into cache

			$key = "$caller_number $called_number $incoming_trunk_group $outgoing_trunk_group $start_time $end_time";

			if($CDR{$key}) {
				# Drop dupes
				printf STDERR "	DUPE DROPPPED!\n";
				syswrite(ZTE, "CHARGEACKOK AID=1\n", 18);
				delete $CDR{$key};
				next;
			}

			$CDR{$key}->{caller_number} = $caller_number;
			$CDR{$key}->{called_number} = $called_number;
			$CDR{$key}->{incoming_trunk_group} = $incoming_trunk_group;
			$CDR{$key}->{outgoing_trunk_group} = $outgoing_trunk_group;
			$CDR{$key}->{start_time} = $start_time;
			$CDR{$key}->{end_time} = $end_time;

			$start_time =~ s/[\:\-]/ /g;
			$end_time =~ s/[\:\-]/ /g;

			($year, $mon, $day, $hour, $min, $sec) = split(' ',$start_time);
			$unix_start_time = timelocal($sec,$min,$hour,$day,$mon-1,$year-1900);
			($year, $mon, $day, $hour, $min, $sec) = split(' ',$end_time);
			$unix_end_time = timelocal($sec,$min,$hour,$day,$mon-1,$year-1900);
		
			$CDR{$key}->{unix_start_time} = $unix_start_time;
			$CDR{$key}->{unix_end_time} = $unix_end_time;

			printf STDERR "	Sent to STDOUT!\n";

			print $CDR{$key}->{unix_start_time}.",".$CDR{$key}->{start_time}.",".($CDR{$key}->{unix_end_time}-$CDR{$key}->{unix_start_time}).",".($CDR{$key}->{unix_end_time}-$CDR{$key}->{unix_start_time}).",".$CDR{$key}->{caller_number}.",".$CDR{$key}->{called_number}.",".$CDR{$key}->{incoming_trunk_group}.",".$CDR{$key}->{outgoing_trunk_group}."\n";

		}
		printf STDERR "\n";

	}


#
#Successful in receiving call tickets: CHARGEACKOK AID =?  
#      Fail in receiving call tickets: CHARGEACKFAIL AID =?  

#Record type 	Byte 	1 	0 	1 : PSTN / IDD / DDD 
#2 : DBO 
#3 : IN 
#4: ISDN/CTX
#5: TAX
#Part record ID 	Byte 	1	1	0: Single record;  
#1: Record (part I);  
#2: Record (middle part);  
#3: Record (last part);  
#Nature address of caller number 	Byte	1	2	0: Subscriber Number;  
#1: Idle;  
#2: Domestic Valid;  
#3: International Valid;  
#Caller number 	Byte 	20 	3 	BCD, left justify, 0xFF as supplementary on the right 
#Nature address of called number 	Byte 	1 	23 	0: Subscriber Number;  
#1: Idle;  
#2: Domestic Valid;  
#3: International Valid;  
#Called number 	Byte 	20 	24 	BCD, left justify, 0xFF as supplementary on the right 
#Start time 	Int 	4 	44 	Unit:second; 
#Start ticks 	Byte 	1 	48 	Unit:10 Millisecond 
#Service category 	Byte 	1 	49 	0 : null 
#1 : intra exchange call 
#2 : Home Network local call 
#3 : Home Network rural service area call 
#4 : Domestic toll call (intra-zone) manual service 
#5 : Domestic toll call (intra-zone) automatic service 
#6 : Domestic toll call ( inter-zone) manual service 
#7 : Domestic toll call (inter-zone) automatic service 
#8 : international long distance manual service 
#9 : international toll call automatic service 
#10 : Home Side charging Special Service 
#11 : Home Side free Special Service 
#12 : Outgoing charging Special Service 
#13 : Outgoing free Special Service 
#14 : Supplementary service 
#15 : CENTREX intra exchange call Service 
#16 : CENTREX Outgoing call Service 
#17 : IN services 
#End time 	Int 	4 	50 	Unit:second; 
#End ticks 	Byte 	1 	54 	Unit:10ms; 
#Release reason 	Byte 	1 	55 	Reason for termination of a call:  
#0: Onhook of the calling party;  
#1: Onhook of the called party;  
#2: Abnormal Disconnection;  
#3: Unused;  
#Caller type 	Byte 	1 	56 	0xA : Normal subscriber 
#Record valid ID 	Bit 	1 / 8 (Low) 	57 	0: Valid;  
#1: Invalid;  
#Clock ID 	Bit 	1 / 8 	57.125 	0 : Clock change during a call;  
#1 : No clock change during a call;  
#Charge flag 	Bit 	1 / 8 	57.250 	0: Free of charge;  
#1: Metering;  
#Attempt call ID 	Bit 	1 / 8 	57.375 	0: Free attempt;  
#1: Charging attempt;  
#Caller analyze ID 	Bit 	1 / 8 	57.500 	0 : Calling number in the call ticket as the charging calling number;  
#1 : Called number in the call ticket as the charging Called Number;  
#Called analyze ID 	Bit 	1 / 8 ( high) 	57.625 	0 : Called Number in the call ticket as the charging Called Number;  
#1 : Connecting number in the call ticket as the charging Called Number;  
#Reserved 	Bit 	1 / 4 	57.750 	
#Incoming trunk group 	Word 	2 	58 	
#Outgoing trunk group 	Word 	2 	60 	
#Supplementary service 	Byte 	7 	62 	7*8 bit fields means 56 kinds of Supplementary Services use Information 
#Charge party ID 	Byte 	1 	69 	1: calling number;  
#2: Called Number;  
#3: Destination address charging;  
#127 : Specified number charging;  
#Nature address of link number 	Byte 	1 	70 	0: Subscriber Number;  
#1: Idle;  
#2: Domestic valid;  
#3: International valid;  
#Link number 	Byte 	20 	71 	BCD, left justify, 0xFF as supplementary on the right 
#Fee 	Int 	4 	91 	
#Bearer services 	Byte 	1 	95 	0: not assigned;  
#1 64K unlimited Digital information 
#2: Voice;  
#3: 3.1KHz Voice Frequency;  
#4 : grouping bearer service;  
#5 : subscriber signaling bearer service;  
#6 : unused;  
#7 : 2X64kbps unlimited;  
#8 : 384kbps unlimited;  
#9 : 1536kbps unlimited;  
#10 : 1920kbps unlimited 
#Terminal services 	Byte 	1 	96 	0: not assigned;  
#1: teleservice;  
#2 : Category 2 / 3 Facsimile;  
#3 : Category 4 Facsimile;  
#4 : Teletex;  
#5 : Videotex;  
#6 :  Videophone;  
#7 : Telex;  
#UUS1 	Byte 	1 	97 	
#UUS3 	Byte 	1 	98 	
#Caller special number 	Byte 	5 	99 	BCD, left justify, 0xFF as supplementary on the right 
#Called special number 	Byte 	5 	104 	BCD, left justify, 0xFF as supplementary on the right 
#Centrex group ID 	Word 	2 	109 	
#Nature address of billed number 	Byte 	1 	111 	0: Subscriber Number;  
#1: Idle;  
#2: Domestic valid;  
#3: International valid;  
#Billed number 	Byte 	11 	112 	BCD, left justify, 0xFF as supplementary on the right 
#
