#!/usr/bin/perl
#
# This script peers into inet_fixed_ip_services for disable services and
# blocks their CIDRs by forming an ip access-list. 
#

$ACL_ID = "123"; ###  Extended Access-List number 

$ACL_HEAD{'CISCO'} = "no ip access-list ext $ACL_ID \n".
       		     "ip access-list ext $ACL_ID \n".
       		     " remark Automagically generated by CHERRY\n";
$ACL_FOOT{'CISCO'} = " permit ip any any\n";
$ACL_HEAD{'UNIX'}  = "sudo ipfw delete $ACL_ID\n";
$ACL_FOOT{'UNIX'}  = " \n";

$BASE = "$0";
$BASE =~ s/\/[a-zA-Z0-9\._\-]*$//;
$config .= "//etc/cherry/cherry.cfg";

require "$BASE//etc/cherry/cherry.cfg";
require "/usr/lib/cherry/dbaccess";


############################################################################

$sth = $dbh->prepare("
	SELECT distinct(inet_fixed_ip_services.nas_id),
	       base_nas_list.nas_type, 
	       base_nas_list.login, 
	       base_nas_list.access_type 
	FROM inet_fixed_ip_services, 
	     base_nas_list
	WHERE inet_fixed_ip_services.oper_id = base_nas_list.oper_id
	  AND inet_fixed_ip_services.nas_id = base_nas_list.nas_id
	");

$sth -> execute;

while( ($nas_id, $nas_type, $login, $access_type) = $sth->fetchrow_array ) {
	if(! check_nas_type($nas_type)) {
		print STDERR "NAS TYPE: $nas_type is not currently supported! NAS $nas_is skipped!\n";
		next;
	}
	if(! $ACL{$nas_id} ) { 
		$ACL{$nas_id}->{'ACL'} = $ACL_HEAD{$nas_type};
		$ACL{$nas_id}->{'TYPE'} = $nas_type;
		$ACL{$nas_id}->{'ACCESS'} = $access_type;
		$ACL{$nas_id}->{'LOGIN'} = $login;
	}
}


$sth = $dbh->prepare("
	SELECT inet_fixed_ip_services.user_id, 
	       inet_fixed_ip_services.nas_id,
	       base_nas_list.nas_type, 
	       base_nas_list.login, 
	       base_nas_list.access_type 
	FROM inet_fixed_ip_services, 
	     base_nas_list,
	     base_clients
	WHERE inet_fixed_ip_services.oper_id = base_nas_list.oper_id
	  AND inet_fixed_ip_services.nas_id = base_nas_list.nas_id
	  AND inet_fixed_ip_services.client_id = base_clients.client_id
	  AND inet_fixed_ip_services.oper_id = base_clients.oper_id
	  AND (inet_fixed_ip_services.enabled < 1 OR 
	       inet_fixed_ip_services.water_mark >= base_clients.balance) 
	");

$sth -> execute;

while( ($cidr, $nas_id, $nas_type, $login, $access_type) = $sth->fetchrow_array ) {

print "$cidr at $nas_id, type = $access_type\n";
	if(! check_nas_type($nas_type)) {
		print STDERR "NAS TYPE: $nas_type is not currently supported! NAS $nas_is skipped!\n";
		next;
	}
	if(! $ACL{$nas_id} ) { 
		$ACL{$nas_id}->{'ACL'} = $ACL_HEAD{$nas_type};
		$ACL{$nas_id}->{'TYPE'} = $nas_type;
		$ACL{$nas_id}->{'ACCESS'} = $access_type;
		$ACL{$nas_id}->{'LOGIN'} = $login;
	}

        $ACL{$nas_id}->{'ACL'} .= &{ "ACL_$nas_type" } ($cidr);

}

foreach $nas (keys %ACL) 
	{ $ACL{$nas}->{'ACL'} .= $ACL_FOOT{ $ACL{$nas}->{'TYPE'}}; }



foreach $nas (keys %ACL) {

	print "ACCESS LIST FOR: $nas, TYPE: ". $ACL{$nas}->{'TYPE'} ."\n";


	open(OUT,"| $BASE/access_nas.sh $CFG::UNIXSHELL_ACCESS_TYPES{$ACL{$nas}->{'ACCESS'}}->{'program'} $nas $ACL{$nas}->{'LOGIN'} $ACL{$nas}->{'ACCESS'} $ACL{$nas}->{'TYPE'} $BASE/../../etc/identity-$nas");

	print OUT $ACL{$nas}->{'ACL'} ."\n";
	print $ACL{$nas}->{'ACL'} ."\n"; 

	close(OUT);
}

exit;

#
sub check_nas_type {
	my ($nas_typ) = shift @_;
	foreach (keys %ACL_HEAD) {
		return 1 if($nas_type eq $_);
	}
	return 0;
}

# ACL_CISCO($cirds) - this function converts a string of standard CIDRs into
#                        CISCO's standard access-list format. 
#
sub ACL_CISCO {
	local($cidrs) = shift @_;
	local($acl)   = "";
	local($cidr);
	local($net);
	local($mask);
	local($bits);

	foreach $cidr (split(' ', $cidrs)) {

		if($cidr =~ /\//) {
			($net, $bits) = split('/', $cidr);

			SWITCH: {
				if ($bits == 0) { $mask = "255.255.255.255"; 
					last SWITCH; }
				if ($bits == 1) { $mask = "127.255.255.255"; 
					last SWITCH; }
				if ($bits == 2) { $mask = "63.255.255.255"; 
					last SWITCH; }
				if ($bits == 3) { $mask = "31.255.255.255"; 
					last SWITCH; }
				if ($bits == 4) { $mask = "15.255.255.255"; 
					last SWITCH; }
				if ($bits == 5) { $mask = "7.255.255.255"; 
					last SWITCH; }
				if ($bits == 6) { $mask = "3.255.255.255"; 
					last SWITCH; }
				if ($bits == 7) { $mask = "1.255.255.255"; 
					last SWITCH; }
				if ($bits == 8) { $mask = "0.255.255.255"; 
					last SWITCH; }
				if ($bits == 9) { $mask = "0.127.255.255"; 
					last SWITCH; }
				if ($bits ==10) { $mask = "0.63.255.255"; 
					last SWITCH; }
				if ($bits ==11) { $mask = "0.31.255.255"; 
					last SWITCH; }
				if ($bits ==12) { $mask = "0.15.255.255"; 
					last SWITCH; }
				if ($bits ==13) { $mask = "0.7.255.255"; 
					last SWITCH; }
				if ($bits ==14) { $mask = "0.3.255.255"; 
					last SWITCH; }
				if ($bits ==15) { $mask = "0.1.255.255"; 
					last SWITCH; }
				if ($bits ==16) { $mask = "0.0.255.255"; 
					last SWITCH; }
				if ($bits ==17) { $mask = "0.0.127.255"; 
					last SWITCH; }
				if ($bits ==18) { $mask = "0.0.63.255"; 
					last SWITCH; }
				if ($bits ==19) { $mask = "0.0.31.255"; 
					last SWITCH; }
				if ($bits ==20) { $mask = "0.0.15.255"; 
					last SWITCH; }
				if ($bits ==21) { $mask = "0.0.7.255"; 
					last SWITCH; }
				if ($bits ==22) { $mask = "0.0.3.255"; 
					last SWITCH; }
				if ($bits ==23) { $mask = "0.0.1.255"; 
					last SWITCH; }
				if ($bits ==24) { $mask = "0.0.0.255"; 
					last SWITCH; }
				if ($bits ==25) { $mask = "0.0.0.127"; 
					last SWITCH; }
				if ($bits ==26) { $mask = "0.0.0.63"; 
					last SWITCH; }
				if ($bits ==27) { $mask = "0.0.0.31"; 
					last SWITCH; }
				if ($bits ==28) { $mask = "0.0.0.15"; 
					last SWITCH; }
				if ($bits ==29) { $mask = "0.0.0.7"; 
					last SWITCH; }
				if ($bits ==30) { $mask = "0.0.0.3"; 
					last SWITCH; }
				if ($bits ==31) { $mask = "0.0.0.1"; 
					last SWITCH; }
				$mask = "0.0.0.0";
			}

			### $acl = $acl . " deny ip any $net $mask\n";	
			$acl = $acl . " deny ip $net $mask any\n";	

		} else {
			### $acl = $acl . " deny ip any host $cidr\n";	
			$acl = $acl . " deny ip host $cidr any\n";	
		}
	}

	return $acl;
}

# ACL_UNIX($cirds) - this function converts a string of standard CIDRs into
#                        a UNIX IPFW list format. 
#
sub ACL_UNIX {
	local($cidrs) = shift @_;
	local($acl)   = "";
	local($cidr);
	local($net);
	local($mask);
	local($bits);

	foreach $cidr (split(' ', $cidrs)) {

		if($cidr =~ /\//) {
			($net, $bits) = split('/', $cidr);

			SWITCH: {
				if ($bits == 0) { $mask = "0.0.0.0"; 
					last SWITCH; }
				if ($bits == 1) { $mask = "128.0.0.0"; 
					last SWITCH; }
				if ($bits == 2) { $mask = "192.0.0.0"; 
					last SWITCH; }
				if ($bits == 3) { $mask = "224.0.0.0"; 
					last SWITCH; }
				if ($bits == 4) { $mask = "240.0.0.0"; 
					last SWITCH; }
				if ($bits == 5) { $mask = "248.0.0.0"; 
					last SWITCH; }
				if ($bits == 6) { $mask = "252.0.0.0"; 
					last SWITCH; }
				if ($bits == 7) { $mask = "254.0.0.0"; 
					last SWITCH; }
				if ($bits == 8) { $mask = "255.0.0.0"; 
					last SWITCH; }
				if ($bits == 9) { $mask = "255.128.0.0"; 
					last SWITCH; }
				if ($bits ==10) { $mask = "255.192.0.0"; 
					last SWITCH; }
				if ($bits ==11) { $mask = "255.224.0.0"; 
					last SWITCH; }
				if ($bits ==12) { $mask = "255.240.0.0"; 
					last SWITCH; }
				if ($bits ==13) { $mask = "255.248.0.0"; 
					last SWITCH; }
				if ($bits ==14) { $mask = "255.252.0.0"; 
					last SWITCH; }
				if ($bits ==15) { $mask = "255.254.0.0"; 
					last SWITCH; }
				if ($bits ==16) { $mask = "255.255.0.0"; 
					last SWITCH; }
				if ($bits ==17) { $mask = "255.255.128.0"; 
					last SWITCH; }
				if ($bits ==18) { $mask = "255.255.192.0"; 
					last SWITCH; }
				if ($bits ==19) { $mask = "255.255.224.0"; 
					last SWITCH; }
				if ($bits ==20) { $mask = "255.255.240.0"; 
					last SWITCH; }
				if ($bits ==21) { $mask = "255.255.248.0"; 
					last SWITCH; }
				if ($bits ==22) { $mask = "255.255.252.0"; 
					last SWITCH; }
				if ($bits ==23) { $mask = "255.255.254.0"; 
					last SWITCH; }
				if ($bits ==24) { $mask = "255.255.255.0"; 
					last SWITCH; }
				if ($bits ==25) { $mask = "255.255.255.128"; 
					last SWITCH; }
				if ($bits ==26) { $mask = "255.255.255.192"; 
					last SWITCH; }
				if ($bits ==27) { $mask = "255.255.255.224"; 
					last SWITCH; }
				if ($bits ==28) { $mask = "255.255.255.240"; 
					last SWITCH; }
				if ($bits ==29) { $mask = "255.255.255.248"; 
					last SWITCH; }
				if ($bits ==30) { $mask = "255.255.255.252"; 
					last SWITCH; }
				if ($bits ==31) { $mask = "255.255.255.254"; 
					last SWITCH; }
				$mask = "255.255.255.255";
			}

			$acl = $acl . " sudo ipfw add $ACL_ID reject ip from any to $net:$mask\n";	

		} else {
			$acl = $acl . " sudo ipfw add $ACL_ID reject ip from any to $cidr\n";	
		}
	}

	return $acl;
}

