#!/usr/bin/perl


$config =  "/etc/cherry/radsrv.cfg";
$flowcachefile = "/var/lib/cherry/radsrv.flow";

$debug   = 2;
$debug_level = 3;
$TIMER   = 5;
$LOGGING = 2; 
$FLOW_CHECK_INTERVAL = 60; # check netflow data each XXX secs
$FLOW_EXPIRE_TIME_ASSOCIATED = 60;
$FLOW_EXPIRE_TIME_UNASSOCIATED = 60 * 15;
$FLOW_TRIGGER_BYTES_ASSOCIATED =  1024 * 500;
$FLOW_TRIGGER_BYTES_UNASSOCIATED = 1024 * 1024 * 10;
$FLOW_REDUCE_BYTES = 1000; # drop flow if it is lower than this value 
$FLOW_REDUCE_BYTES_INPUT = 100;
$USM_TIMEOUT = 125; # remove session from USM if there's no Alive message
$USM_DEADTIME = 15; # Keep session in USM at least for this amount of time before actually removing it 
$RSM_TIMEOUT = 40; # remove request from RSM if it was not responded in 10 sec

$|=1;


require "$config";


    use Time::HiRes;
    use Socket;
    use Sys::Hostname;
    use Fcntl;
    use RADIUS::Dictionary;
    use RADIUS::Packet;
    use FileHandle;




    my ( $LOG_DATA ) = "";

    my ( $hisiaddr, $hispaddr, 
	 $host, $paddr, $port, $proto,
	 $rin, $rout, $rc );

    my ( $stat_netflow_dumps,
	 $stat_auth_requests_received,
	 $stat_auth_requests_respond,
	 $stat_acct_requests_received,
	 $stat_acct_requests_ignored,
	 $stat_netflow_requests_received,
	 $stat_rsms,
	 $flow_hash_size,
	 $flow_last_check ) = (0,0,0,0,0,0,0,0,0);

    my ($CRISIS) = 1; ### initial code cannot be interrupted


    # Parse the RADIUS dictionary file (must have dictionary in current dir)
    my $dict = new RADIUS::Dictionary "$dictionary" || die "Couldn't read dictionary: $!";

    #
    # Preparing local interface to accept UDP datagrams
    #

    $proto = getprotobyname('udp');
    $t_proto = getprotobyname('tcp');
    $p_auth    = getservbyname('radius', 'udp') if(! $p_auth);
	die "Could not find radauth port" if(! $p_auth);
    $p_acct    = getservbyname('radacct', 'udp') if(! $p_acct);
	die "Could not find radacct port" if(! $p_acct);
    $p_netflow = getservbyname('netflow', 'udp') if(! $p_netflow);
	die "Could not find netflow port" if(! $p_netflow);

    $paddr_auth    = sockaddr_in($p_auth, inet_aton($interface)); 
    $paddr_acct    = sockaddr_in($p_acct, inet_aton($interface));
    $paddr_netflow = sockaddr_in($p_netflow, inet_aton($interface));


    ### START 


    system("echo $$ > $pidfile");

    while( 1 ) {
    if(fork == NULL) {

    socket(AUTH, PF_INET, SOCK_DGRAM, $proto)   || die "auth socket: $!";
    bind(AUTH, $paddr_auth)                     || die "auth bind: $!";
    LOG("AUTH socket ($p_auth) bound OK");

    socket(ACCT, PF_INET, SOCK_DGRAM, $proto)   || die "acct socket: $!";
    bind(ACCT, $paddr_acct)                     || die "acct bind: $!";
    LOG("ACCT socket ($p_acct) bound OK");

    socket(NETFLOW, PF_INET, SOCK_DGRAM, $proto)   || die "netflow socket: $!";
    bind(NETFLOW, $paddr_netflow)                     || die "netflow bind: $!";
    LOG("NETFLOW socket ($p_netflow) bound OK");

    fcntl AUTH, &F_SETFL, O_NONBLOCK;
    fcntl ACCT, &F_SETFL, O_NONBLOCK;
    fcntl NETFLOW, &F_SETFL, O_NONBLOCK;


    SOCKET_RECOVER();

    if(!@tariff_sockets || !@tariff_sockets_for_auth) {
	LOG("No tariffication servers available. Stop!");
	LOG_FLUSH();
	sleep 1;
	exit;
    }

    USM_LOAD();

    FLOW_LOAD();

    foreach (keys %RSM) { RSM_REMOVE($_); }
    LOG("Request State Machine initialized");

    $SIG{'TERM'} = 'term_handler';
    $SIG{'HUP'} = 'hup_handler';
    $SIG{'INT'} = 'term_handler';
    ##$SIG{'CONT'} = 'RSM_DISPLAY';
    $SIG{'ALRM'} = 'timer_handler';
    $SIG{'PIPE'} = 'pipe_handler';

    LOG(sprintf "RADIUS Server started");

    alarm($TIMER);

    while ( 1 ) {

	LOG("Global delay = ".sprintf("%0.04f", Time::HiRes::time - $global_time));
	while( 1 ) {
	  LOG_FLUSH(1);
	  $CRISIS = 0;  ### this code can be interruped
	  $s = select($rout = $rin, undef, undef, 5);
	  ### LOG("Select = $s");
	  if($s > 0) { last; } 
	  if($s <0) {LOG("Select error: $!");}
	}

	$CRISIS = 1;
	$data = '';


	$global_time = Time::HiRes::time;

	if($hispaddr = recv(ACCT, $data, 1024, 0)) {
		LOG(sprintf "*** Datum on ACCT, packet size = %d", length($data)) if($debug_level > 0);
		process_nas_request(ACCT, $data, $hispaddr);
	}

	if ($hispaddr = recv(AUTH, $data, 1024, 0)) {
		LOG(sprintf "*** Datum on AUTH, packet size = %d", length($data)) if($debug_level > 0);
		process_nas_request(AUTH, $data, $hispaddr);
	}

	if ($hispaddr = recv(NETFLOW, $data, 10240, 0)) {
		LOG(sprintf "*** Datum on NETFLOW, packet size = %d", length($data)) if($debug_level > 0);
		process_netflow(NETFLOW, $data, $hispaddr);
	}


	foreach my $sock (@tariff_sockets) {
		if(vec($rout, fileno($sock), 1)) {
			if(sysread($sock, $data, 16768)) {
				LOG(sprintf "*** Datum on $_ from tariff, packet size = %d", length($data)) if($debug_dlevel > 0);
				process_tarif_response($data);
			} else {
				SOCKET_REMOVE($sock);
			}
		}
	}

	foreach my $sock (@tariff_sockets_for_auth) {
		if(vec($rout, fileno($sock), 1)) {
			if(sysread($sock, $data, 16768)) {
				LOG(sprintf "*** Datum on $_ from auth, packet size = %d", length($data)) if($debug_dlevel > 0);
				process_tarif_response($data);
			} else {
				SOCKET_REMOVE_FROM_AUTH($sock);
			}
		}
	}


	undef $data;
    }


    ### END OF FORK
    } 

    wait;
    sleep 1;
    ### END OF WHILE 
    }


sub SOCKET_RECOVER {

    #
    # Preparing tariffication sockets, or recover those not connected yet. 
    #

    foreach (@tariff_servers) {
	next if (join(" ", @tariff_sockets) =~ /$_/);

	### print("Trying tariff server: $_\n");
	my($t_host, $t_port) = split(':'), 
    	$t_proto = getprotobyname('tcp');
    	$t_paddr_auth = sockaddr_in($t_port, inet_aton($t_host)); 
	$TARIFF = "$t_host:$t_port #".(int rand(1000));
	if(socket($TARIFF, PF_INET, SOCK_STREAM, $t_proto)) {
		### print ("\tsocket allocated\n");
	} else {
		### print("\tsocket() failed: $!\n");
		next;
	}


	if(connect($TARIFF, $t_paddr_auth) ) {
		### print ("\tsocket connected\n");
	} else {
		### print("\tconnect() failed: $!\n");
		close($TARIFF);
		next;
	}

    	fcntl $TARIFF, &F_SETFL, O_NONBLOCK;
	$TARIFF->autoflush(1);
	push @tariff_sockets, $TARIFF;
	LOG("Discovered tariffication socket: $TARIFF");
    }


    ### Prioritized sockets for AUTH requests

    foreach (@tariff_servers_for_auth) {
	next if (join(" ", @tariff_sockets_for_auth) =~ /$_/);

	### print("Trying tariff server: $_\n");
	my($t_host, $t_port) = split(':'), 
    	$t_proto = getprotobyname('tcp');
    	$t_paddr_auth = sockaddr_in($t_port, inet_aton($t_host)); 
	$TARIFF = "$t_host:$t_port #".(int rand(1000));
	if(socket($TARIFF, PF_INET, SOCK_STREAM, $t_proto)) {
		### print ("\tsocket allocated\n");
	} else {
		### print("\tsocket() failed: $!\n");
		next;
	}


	if(connect($TARIFF, $t_paddr_auth) ) {
		### print ("\tsocket connected\n");
	} else {
		### print("\tconnect() failed: $!\n");
		close($TARIFF);
		next;
	}

    	fcntl $TARIFF, &F_SETFL, O_NONBLOCK;
	$TARIFF->autoflush(1);
	push @tariff_sockets_for_auth, $TARIFF;
	LOG("Discovered prioritized tariffication socket: $TARIFF");
    }

    SOCKET_SELECT();
}

sub SOCKET_SELECT {

    $rin = '';
    vec($rin, fileno(AUTH), 1) = 1;
    vec($rin, fileno(ACCT), 1) = 1;
    vec($rin, fileno(NETFLOW), 1) = 1;
    foreach (@tariff_sockets) { vec($rin, fileno($_), 1) = 1; }
    foreach (@tariff_sockets_for_auth) { vec($rin, fileno($_), 1) = 1; }

}

sub SOCKET_REMOVE {
	my $sock = shift @_;
	my @new_tariff_sockets = ();
	foreach (@tariff_sockets) {
		push(@new_tariff_sockets, $_) if(!($sock eq $_));
	}
	@tariff_sockets = @new_tariff_sockets;
	close($sock);
	SOCKET_SELECT();

	LOG("Socket $sock removed from ring");
}

sub SOCKET_REMOVE_FROM_AUTH {
	my $sock = shift @_;
	my @new_tariff_sockets = ();
	foreach (@tariff_sockets_for_auth) {
		push(@new_tariff_sockets, $_) if(!($sock eq $_));
	}
	@tariff_sockets_for_auth = @new_tariff_sockets;
	close($sock);
	SOCKET_SELECT();

	LOG("Socket $sock removed from auth ring");
}

sub process_tarif_response {
	my($data) 	= shift @_;

	foreach (split('\n', $data)) {
		next if(! (/^OK/ || /^ERROR/));

		LOG("Tarifficator respond: $_");

		my (	$code, 
	    		$id,
	    		$credit_amount,
	    		$credit_time,
	    		$currency,
	    		$gender_h,
	    		$gender_l,
			$nas_type,
			$option_features,
	    		$option_ip_address,
	    		$option_ip_netmask,
	    		$option_ip_routing,
	    		$option_ip_filter_id,
	    		$option_max_connections,
	    		$mschap2_authenticator_response,
		) = split('\t');

		$id =~ s/\:.*//g; ### ID = salt@host:time 

		if($code =~ /ERROR/ && $USM{$id}) {
			LOG("Negative response, $id MUST be disconnected!");
			DISCONNECT($id);
		}

		if(! defined($RSM{$id}) ) {
			LOG("No RSM, response skipped");
			next;
		}

	  	my $rp = new RADIUS::Packet $dict;

	  	$rp->set_identifier($RSM{$id}->{'Request'}->identifier);
	  	$rp->set_authenticator($RSM{$id}->{'Request'}->authenticator);

		
		if( $code eq 'OK' && 
			$RSM{$id}->{'Request'}->code eq 'Access-Request' ) {

			# it was Access Request in RSM, do make proper response

			# Check max connections first
			my($num_of_ses) = NUMBER_OF_SESSIONS($RSM{$id}->{'Request'}->attr('User-Name'), 
							   $RSM{$id}->{'Service-Type'}); 

			if(!defined($options_max_connection) || 
				($option_max_connections > 0 && 
				$option_max_connections >= ($num_of_ses + 1)) ) {
	  			$rp->set_code('Access-Accept');
				LOG("RSM $id AUTHORIZED, time: $credit_time, amount: $credit_amount, currency_type: $currency, gender: $gender_h:$gender_l, max con = $option_max_connections:$num_of_ses");
			} else {
	  			$rp->set_code('Access-Reject');
				LOG("RSM $id NOT AUTHORIZED: Maximum number of connections reached: current = $num_of_ses, allowed = $option_max_connections");
			}

                	### Send to NAS some VSA attributes...

			if($RSM{$id}->{'Request'}->{MSCHAP2_Ident} ) {
				$rp->set_attr('Vendor-Specific', 
					VSA_PACK($rp,   311, 
					{'MS-CHAP2-Success',  pack("C a42", $RSM{$id}->{'Request'}->{MSCHAP2_Ident}, 
						$mschap2_authenticator_response)} ));
					LOG("MSCHAP2 AuthentResponse: ".$rp->attr('MS-CHAP2-Success'));
			}


LOG("RAD: IP-Filter: $option_ip_filter_id, Framed-Routing: $option_ip_routing");
			$rp->set_attr('Framed-Protocol', 
				$RSM{$id}->{'Request'}->attr('Framed-Protocol')) 
				if($RSM{$id}->{'Request'}->attr('Framed-Protocol'));
			$rp->set_attr('Framed-IP-Address',$option_ip_address) 
				if($option_ip_address);
			$rp->set_attr('Framed-IP-Netmask',$option_ip_netmask) 
				if($option_ip_netmask);
			$rp->set_attr('Framed-Routing',$option_ip_routing) 
				if($option_ip_routing);
			$rp->set_attr('Filter-Id',$option_ip_filter_id) 
				if($option_ip_filter_id);

			$rp->set_attr('Idle-Timeout', 7200)
				if($nas_type =~ /ASCEND/);

			if($RSM{$id}->{'Service-Type'} =~ /voip/ ||
			   $RSM{$id}->{'Service-Type'} =~ /telephone_info/ ||
			   $RSM{$id}->{'Service-Type'} =~ /card/) {
				### $gender_h and $gender_l do set up CISCO to
				### tell user his account correctly.
                		### 0 - male, 1 - female

				$rp->set_attr('Vendor-Specific', VSA_PACK($rp,   9, {'Cisco-AVPair',"h323-credit-time=$credit_time $gender_h"}));
				$rp->set_attr('Vendor-Specific', VSA_PACK($rp,   9, {'Cisco-AVPair',"h323-credit-amount=$credit_amount $gender_h $gender_l $currency"}));
			}

			if($RSM{$id}->{'Service-Type'} =~ /inet/ && $option_features =~ /CALLBACK/) {
				# Is it a callback request ???

				if($nas_type =~ /CISCO/) {
					$rp->set_attr('Service-Type','Framed-User'); 
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   9, 
							{'Cisco-AVPair',"lcp:callback-dialstring="}));
				} elsif ($nas_type =~ /ASCEND/) {
					$rp->set_attr('Service-Type','Callback-Framed-User'); 
					#$rp->set_attr('Callback-Number',''); 
					#$rp->set_attr('Service-Type','Framed-User'); 
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   529, 
							{'Ascend-CBCP-Enable',"CBCP-Enabled"}));
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   529, 
							{'Ascend-CBCP-Mode',"CBCP-User-Callback"}));
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   529, 
							{'Ascend-CBCP-Trunk-Group',9}));
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   529, 
							{'Ascend-Data-SVC','Switched-Modem'}));
					$rp->set_attr('Vendor-Specific', 
						VSA_PACK($rp,   529, 
							{'Ascend-Send-Auth','Send-Auth-None'}));
				} else {
					$rp->set_attr('Service-Type','Callback-Framed-User'); 
				}
			} else {

				$rp->set_attr('Service-Type',
				$RSM{$id}->{'Request'}->attr('Service-Type'));
			}
					


		} elsif ($code eq 'ERROR' &&
			$RSM{$id}->{'Request'}->code eq 'Access-Request') {

			LOG("RSM $id NOT AUTHORIZED");
	  		$rp->set_code('Access-Reject');

		} else {
			# do nothig yet, some unknown type of RSM it was
		}

    		my $his = sockaddr_in($RSM{$id}->{'Port'}, 
			inet_aton($RSM{$id}->{'Host'})); 

	  	send(AUTH, auth_resp($rp->pack, $radius_client{$RSM{$id}->{'Host'}}), 0, $his);

		RSM_REMOVE($id);
		$stat_auth_requests_respond += 1;
	}

}


sub process_nas_request {
	my($SOCK)   = shift @_;
	my($packet) = shift @_;
	my($his)    = shift @_;

	($port, $hisiaddr) = sockaddr_in($his);
	##$host = gethostbyaddr($hisiaddr, AF_INET);
	$host = inet_ntoa($hisiaddr);
	my($cur_time)  = time;
	my($secret)    = $radius_client{$host};

	#MS-CHAPv2
	my($MSCHAP2_Ident, $MSCHAP2_PeerChallenge,$MSCHAP2_Response, $MSCHAP2_Challenge);
 
	
	LOG("*** His addr: $host, port: $port") if($debug_level > 0);

	if(! $secret) {
           LOG("Host is not listed as a client, skip request");	
	   return;
	}

        my $p = new RADIUS::Packet $dict, $packet;


### REMOVE CODE BELOW ###
                        foreach (
                                grep /h323/,
                                values %{$p->{Attributes}} ) {
                                        LOG("DEBUG: $_");
                                        if(/hack:true/)
                                                ### THIS IS A HACK 
                                                { $host = "hack_".$host; last; }
                        }

### REMOVE CODE ABOVE ###

        if ($p->code eq 'Access-Request') {

         ### Process access request...


          LOG(sprintf "Access-Request: %s:%d", $host, $port) if($debug_level > 0); 
          LOG(sprintf "\tIdentifier = \"%s\"", $p->identifier) if($debug_level > 0); 
          LOG(sprintf "\tUser-Name = \"%s\"", $p->attr('User-Name')) if($debug_level > 0); 
          LOG(sprintf "\tPassword = \"%s\"", hex_dump_print($p->password($secret))) if($debug_level > 0); 
          LOG(sprintf "\tService-Type = \"%s\"", $p->attr('Service-Type')) if($debug_level > 0); 
          LOG(sprintf "\tFramed-Protocol = \"%s\"", $p->attr('Framed-Protocol')) if($debug_level > 0); 
          LOG(sprintf "\tFramed-IP-Address = \"%s\"", $p->attr('Framed-IP-Address')) if($p->attr('Framed-IP-Address') && $debug_level > 0); 
          LOG(sprintf "\tCalling-Station-Id = \"%s\"", $p->attr('Calling-Station-Id')) if($debug_level > 0); 
          LOG(sprintf "\tCalled-Station-Id = \"%s\"", $p->attr('Called-Station-Id')) if($debug_level > 0); 

	  foreach my $attr ($p->attributes) {
		next if (! ($attr =~ /Vendor/) );
		LOG("\t$attr = ".hex_dump_print($p->attr($attr))) if($debug_level > 0);
		my($vsa_vendor, $vsa_type, $vsa_length, $vsa_data ) = unpack("N C C a*", $p->attr($attr));
		LOG("\t\tvendor = $vsa_vendor,\tType = $vsa_type,\tLength = $vsa_length\tData   = ".hex_dump_print($vsa_data)) if($debug_level > 0);

		if($vsa_vendor == 311 && $vsa_type == 25) {
			($MSCHAP2_Ident, $_, $MSCHAP2_PeerChallenge, $_, $MSCHAP2_Response) = unpack("C C a16 a8 a24", $vsa_data);
			$p->{$MSCHAP2_PeerChallenge} = $MSCHAP2_PeerChallenge = hex_dump($MSCHAP2_PeerChallenge);
			$p->{MSCHAP2_Response} = $MSCHAP2_Response = hex_dump($MSCHAP2_Response);
			$p->{MSCHAP2_Ident} = $MSCHAP2_Ident;
			LOG("\t\tMS-CHAP-v2: PeerChallenge = $MSCHAP2_PeerChallenge, PeerResponse = $MSCHAP2_Response") if($debug_level > 0);
		}
		if($vsa_vendor == 311 && $vsa_type == 11) {
			$p->{$MSCHAP2_Challenge} = 
				$MSCHAP2_Challenge = hex_dump($vsa_data);
			LOG("\t\tMS-CHAP-v2: AuthChallenge = $MSCHAP2_Challenge") if($debug_level > 0);
		}
	  }

	  return if (! RSM_ADD($p, $host, $port) ); 

### CISCO_NAS_PORT($p);

          SEND_TO_TARIF_SERVER(
		"auth",				# CMD
                $p->identifier.'@'.$host.':'.$cur_time, # "$sid@$host:time"
                SERVICE_TYPE($p),               # service type (voip_ip ... )
                $cur_time,                      # current time
                60,                             # session long = 60 secs
                $p->attr('User-Name'),          # user's id
                #"321",		#
                SERVICE_TYPE($p) eq 'voip_gk' ? # # is this a VoIP Gatekeeper auth atempt ? 
				$p->attr('Framed-IP-Address') :  
			$MSCHAP2_PeerChallenge ?  # is this an MSCHAPv2 auth attempt ? 
					"$MSCHAP2_PeerChallenge:$MSCHAP2_Response:$MSCHAP2_Challenge" :
			$p->password($secret),	  # nah... just plain password
                $host,				# NAS id
                $p->attr('Calling-Station-Id'), # src major
                $p->attr('Calling-Port-Id'), 	# src major
                $p->attr('Called-Station-Id'),  # dst major
                $p->attr('Called-Port-Id'),  	# dst major
		1024 * 1024,
		1024 * 1024,
		REMOTE_MEDIA_ADDRESS($p),
		DISCONNECT_CAUSE($p),
		VOICE_QUALITY($P)
          );

	  $stat_auth_requests_received += 1;

       } elsif ($p->code eq 'Accounting-Request') {

	### Process accounting request...

          LOG(sprintf "Accounting-Request: %s:%d", $host, $port) if($debug_level > 0); 
          LOG(sprintf "\tIdentifier = \"%s\"", $p->identifier) if($debug_level > 0); 
          LOG(sprintf "\tAcct-Session-Id = \"%s\"", $p->attr('Acct-Session-Id')) if($debug_level > 0); 
          LOG(sprintf "\tAcct-Status-Type = \"%s\"", $p->attr('Acct-Status-Type')) if($debug_level > 0); 
          LOG(sprintf "\tUser-Name = \"%s\"", $p->attr('User-Name')) if($debug_level > 0); 
          LOG(sprintf "\tService-Type = \"%s\"", $p->attr('Service-Type')) if($debug_level > 0); 
          LOG(sprintf "\tFramed-Protocol = \"%s\"", $p->attr('Framed-Protocol')) if($debug_level > 0); 
          LOG(sprintf "\tFramed-IP-Address = \"%s\"", $p->attr('Framed-IP-Address')) if($debug_level > 0); 
          LOG(sprintf "\tCalling-Station-Id = \"%s\"", $p->attr('Calling-Station-Id')) if($debug_level > 0); 
          LOG(sprintf "\tCalled-Station-Id = \"%s\"", $p->attr('Called-Station-Id')) if($debug_level > 0); 
          LOG(sprintf "\tAcct-Session-Time = \"%s\"", $p->attr('Acct-Session-Time')) if($debug_level > 0); 
          LOG(sprintf "\tAcct-Delay-Time = \"%s\"", $p->attr('Acct-Delay-Time')) if($debug_level > 0);
          LOG(sprintf "\tNAS-Port-Id = \"%s\"", $p->attr('NAS-Port-Id')) if($debug_level > 0);
          LOG(sprintf "\tAcct-Input-Octets = \"%s\"", $p->attr('Acct-Input-Octets')) if($debug_level > 0);
          LOG(sprintf "\tAcct-Output-Octets = \"%s\"", $p->attr('Acct-Output-Octets')) if($debug_level > 0);
          LOG(sprintf "\tAcct-Terminate-Cause = \"%s\"", $p->attr('Acct-Terminate-Cause')) if($debug_level > 0);



	  # Do sanity check...

	  if(! $p->attr('Acct-Session-Id')) {
		LOG("Session ID not set, request skipped.");
		undef $p;
		return;
	  }

	  # Check Acct-Session-Type...

	  if ($p->attr('Acct-Status-Type') eq 'Start') { ### START ###
		# Completely new session...
		my $id = $p->attr('Acct-Session-Id')."\@".$host;
		if( $USM{$id} ) {
			LOG("Weird situation: session $id already present in USM!");
		} else {
			# add new session to USM
			USM_ADD_SESSION($p, $id, $host, $port);
		} 


	  } elsif ($p->attr('Acct-Status-Type') eq 'Alive') { ### ALIVE ###

		my $id = $p->attr('Acct-Session-Id')."\@".$host;
		if( $USM{$id} ) {
			USM_UPDATE_SESSION($p, $id, $host, $port, 'update');
		} else {
			LOG("Session $id is missing in USM.");
			USM_ADD_SESSION($p, $id, $host, $port);
		}

	  } elsif ($p->attr('Acct-Status-Type') eq 'Stop') { ### STOP ###

		my $id = $p->attr('Acct-Session-Id')."\@".$host;
		if( $USM{$id} ) {
			USM_MARKDEAD_SESSION($p, $id, $host, $port);
			### USM_REMOVE_SESSION($p, $id, $host, $port);
		} else {
			LOG("Session $id is missing in USM. Skipped!");
		}

	  } else {			### WEIRD ACCOUNTING REQUEST ###
		# Buggy session type
		LOG("Buggy session type: ".$p->attr('Acct-Status-Type'));
	  }

	  my $rp = new RADIUS::Packet $dict;

	  $rp->set_identifier($p->identifier);
	  $rp->set_authenticator($p->authenticator);
	  $rp->set_code('Accounting-Response');

	  send($SOCK, auth_resp($rp->pack, $secret) , 0, $his);

	  LOG("Accounting response sent");

  	  $stat_acct_requests_received += 1;

       } else {

	### Process request of unknown type...

          LOG(sprintf "Unknown-Request: %s:%d", $host, $port); 

  	  $stat_acct_requests_ignored += 1;
       }

}


#########################################################################
#########################################################################
#########################################################################

sub LOG {
	my($logstr);
	my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
		localtime(time);
	my($tt) = Time::HiRes::time;

	return if(! $LOGGING );

	my($date) = sprintf "$tt %02d.%02d.%04d %02d:%02d:%02d", $mday, $mon+1, $year+1900, $hour, $min, $sec;

	foreach $logstr (@_) {
		$LOG_DATA = 
		$LOG_DATA . 
		"$date\t$logstr\n";
	}
}

sub LOG_FLUSH {
	if(defined($LOG_DATA) ) {
        	open(LOGFILE, ">>$logfile") || return;
		print LOGFILE $LOG_DATA;
		print STDERR $LOG_DATA if($debug);
		close(LOGFILE);
		$LOG_DATA = "";
	}
}

sub term_handler {
    USM_SAVE();

    foreach $key (keys %FLOW) { FLOW_DUMP_LINE($key); }

    LOG(sprintf "RADIUS Server stoped");
    LOG_FLUSH;
    exit;
}

sub hup_handler {

    LOG("RADIUS Server received HANGUP signal");
    undef $radius_client;
    require "$config";
    LOG("Configuration reloaded");

    if($debug_level++ > 2) {$debug_level = 0;}
}

sub VSA_PACK {
  my ($self) = shift @_;
  my ($vsa_type) = shift @_;
  my (@vsa_list) = @_;

  my $attstr = pack("N", $vsa_type);
  my $p_attr = "C C a*";

  my %packer = ("string" => sub {
                  return $_[0];
                },
                "integer" => sub {
#                 return pack "N", $self->{Dict}->{val}->{$_[1]} ?
                  return pack "N", $self->{Dict}->attr_has_val($_[1]) ?
                    $self->{Dict}->val_num(@_[1, 0]) : $_[0];
                },
                "ipaddr" => sub {
                  return inet_aton($_[0]);
                },
                "time" => sub {
                  return pack "N", $_[0];
                });

  foreach (@vsa_list) {
    my ($attr_name, $attr_val) = %{$_};
    $attr_name =~ s/\..*//;
    ### next if(! $attr_name || ! $attr_var ); ### May cause problems with Cisco
    my $val = &{$packer{$self->{Dict}->attr_type($attr_name)}}($attr_val,
                        $self->{Dict}->attr_num($attr_name));
    $attstr .= pack $p_attr, $self->{Dict}->attr_num($attr_name), length($val)+2, $val;
  }
  return $attstr;
}

sub VSA_CISCO {
	my($i);
	my($vsa);
	my($str);

	$vsa = pack("N", 9);
	foreach $str (@_) {
		$i = length($str);
	 	$vsa = $vsa . pack("CCA$i", 1, $i + 2, $str);
		LOG("VSA: $str");
	}
	LOG("VSA_CISCO: str = '$str', size = ".length($vsa));
	return $vsa;
}


sub pipe_handler {

	LOG("SIGPIPE arrived!");

}

sub timer_handler {
	my($id);
	my($cur_time) = time;

	$timer_delay = Time::HiRes::time;	

	$ticks++;
###	if($ticks >= 10) { 
###
###		$flow_hash_size = 0;
###		foreach (keys %FLOW) {
###			$flow_hash_size += 1;
###		}
###
###		printf "*** STATISTICS: \n";
###		printf "	flow_dumps %.1f\n	flow_hash_size = %d\n	netflow_rq_rcv = %.1f\n	auth_rq_rcv = %.1f\n	auth_rq_rsp = %.1f\n	acct_rq_rcv = %.1f\n	acct_rq_ignored = %.1f\n	RSMs = %d\n", 
###			$stat_netflow_dumps / 10,
###			$flow_hash_size,
###			$stat_netflow_requests_received / 10,
###			$stat_auth_requests_received / 10,
###			$stat_auth_requests_respond / 10,
###			$stat_acct_requests_received / 10,
###			$stat_acct_requests_ignored / 10,
###			$stat_rsms
###		;
###		$stat_netflow_dumps = 0;	
###		$stat_netflow_requests_received = 0;
###		$stat_auth_requests_received = 0;
###		$stat_acct_requests_received = 0;
###		$stat_auth_requests_respond = 0;
###		$stat_acct_requests_ignored = 0;
###		$ticks = 0;
###	}

	if ($CRISIS) {
		alarm($TIMER); ### general process flow is in critical part 
		return;        ### better quit and don't touch anything
	}

$tm1 = Time::HiRes::time;

	# Recover tarification sockets...
	SOCKET_RECOVER();

LOG("Timer 1 = ".sprintf("%0.04f", Time::HiRes::time - $tm1)) if($debug_level >2);
$tm1 = Time::HiRes::time;
$cn1 = 0;

	### Check USM sessions for timeout...
	foreach $id (keys %USM){
		$cn1++;
		if( $cur_time - $USM{$id}->{'Acct-Session-Alive'} > $USM_TIMEOUT) {
			LOG("Session $id (".$USM{$id}->{'User-Name'}.") has expired...");
			USM_REMOVE_SESSION(undef, $id, $USM{$id}->{'NAS-Id'}, undef);
		} elsif ($USM{$id}->{'Dead-Flag'} && 
		   $cur_time - $USM{$id}->{'Acct-Session-Alive'} > $USM_DEADTIME) {

			LOG("Session $id (".$USM{$id}->{'User-Name'}.") is dead...");
			USM_REMOVE_SESSION(undef, $id, $USM{$id}->{'NAS-Id'}, undef);
		}
	}

LOG("Timer 2 = ".sprintf("%0.04f", Time::HiRes::time - $tm1).", cnt = $cn1")  if($debug_level >2);
$tm1 = Time::HiRes::time;

	USM_SAVE();

LOG("Timer 3 = ".sprintf("%0.04f", Time::HiRes::time - $tm1))  if($debug_level >2);
$tm1 = Time::HiRes::time;

	### Check RSM requests for timeout
	foreach $id (keys %RSM) {
		if( $cur_time - $RSM{$id}->{'Start'} > $RSM_TIMEOUT ) {
			LOG(sprintf "Request %s has expired...", 
				$id, $RSM{$id}->{'Host'}, 
				$RSM{$id}->{'Port'});
			RSM_REMOVE($id);
		}
	}

LOG("Timer 4 = ".sprintf("%0.04f", Time::HiRes::time - $tm1))  if($debug_level >2);
$tm1 = Time::HiRes::time;

	FLOW_CHECK();

LOG("Timer 5 = ".sprintf("%0.04f", Time::HiRes::time - $tm1))  if($debug_level >2);
$tm1 = Time::HiRes::time;

	alarm($TIMER);

	LOG("Timer delay = ".sprintf("%0.04f", Time::HiRes::time - $timer_delay))  if($debug_level >2);
}

sub USM_LOAD {
	my($nas_id);
	my($session_id);
	my($__);
	my($id);

	# Load User State Machine persistance from $radusm file...
	open(USM,"$radusm");
	while(<USM>) {
	chop;
	($__, $__, $session_id, $__, $__, $__, 
	 $__, $__, $__, $__, $__, $nas_id, $__) 
		= split(',');

	next if(! $session_id);

	$id = "$session_id\@$nas_id";
	(
		$USM{$id}->{'User-Name'}, 
		$USM{$id}->{'Service-Type'},
		$USM{$id}->{'Acct-Session-Id'}, 
		$USM{$id}->{'Acct-Session-Time'}, 
		$USM{$id}->{'Acct-Session-Alive'}, 
		$USM{$id}->{'Calling-Station-Id'}, 
		$USM{$id}->{'Calling-Port-Id'}, 
		$USM{$id}->{'Called-Station-Id'}, 
		$USM{$id}->{'Called-Port-Id'}, 
		$USM{$id}->{'Acct-Input-Octets'}, 
		$USM{$id}->{'Acct-Output-Octets'}, 
		$USM{$id}->{'NAS-Id'}, 
		$USM{$id}->{'NAS-Port-Id'}, 
		$USM{$id}->{'Framed-IP-Address'}, 
		$USM{$id}->{'Dead-Flag'}, 
	) = split(',');	
	$USM_IP{$USM{$id}->{'Framed-IP-Address'}} = $USM{$id};
	}
	close(USM);
	LOG("User State Machine loaded");
}

sub FLOW_SAVE {

	my($ts) = Time::HiRes::time;
	open(FLOWFH, ">$flowcachefile");
	foreach (keys %FLOW) {
		($ip_host,$ip_src,$ip_dst) = split(' ',$_);
		print FLOWFH $FLOW_TIME{$_}."\t$ip_host\t$ip_src\t$ip_dst\t".$FLOW{$_}."\n";
	}
	close(FLOWFH);
	$ts = Time::HiRes::time - $ts; 
	LOG("FLOW_SAVE: $ts secs");
}

sub FLOW_LOAD {

	open(FLOWFH, "$flowcachefile");
	while(<FLOWFH>) {
		chop;
		my ($ts, $ip_host, $ip_src, $ip_dst, $ip_bytes) = split('\t');
		$_ = "$ip_host $ip_src $ip_dst";
		$FLOW_TIME{$_} = $ts;
		$FLOW{$_} = $ip_bytes;
		print "$ip_host	$ip_src	$ip_dst	$ip_bytes\n";
			
	}
	close(FLOWFH);
}

sub USM_SAVE {
	# Save USM data to $radusm file... Persistance rocks!
	open(USM, ">$radusm");
	foreach $id (keys %USM) {
	print USM 
		$USM{$id}->{'User-Name'},",", 
		$USM{$id}->{'Service-Type'},",",
		$USM{$id}->{'Acct-Session-Id'},",", 
		$USM{$id}->{'Acct-Session-Time'},",", 
		$USM{$id}->{'Acct-Session-Alive'},",", 
		$USM{$id}->{'Calling-Station-Id'},",", 
		$USM{$id}->{'Calling-Port-Id'},",", 
		$USM{$id}->{'Called-Station-Id'},",", 
		$USM{$id}->{'Called-Port-Id'},",", 
		$USM{$id}->{'Acct-Input-Octets'},",", 
		$USM{$id}->{'Acct-Output-Octets'},",", 
		$USM{$id}->{'NAS-Id'},",", 
		$USM{$id}->{'NAS-Port-Id'},",", 
		$USM{$id}->{'Framed-IP-Address'},",", 
		$USM{$id}->{'Dead-Flag'},",", 
	"\n";
	}	
	close(USM);
}

sub USM_ADD_SESSION {
	my($p)    = shift @_;
	my($id)   = shift @_;
	my($host) = shift @_;
	my($port) = shift @_;

	### CISCO_NAS_PORT($p);
			$USM{$id}->{'User-Name'} = $p->attr('User-Name');
			$USM{$id}->{'Service-Type'} = SERVICE_TYPE($p);
			$USM{$id}->{'Acct-Session-Id'} = $p->attr('Acct-Session-Id');
			$USM{$id}->{'Acct-Session-Time'} = time - $p->attr('Acct-Delay-Time');
			$USM{$id}->{'Acct-Session-Alive'} = time - $p->attr('Acct-Delay-Time');
			$USM{$id}->{'Calling-Station-Id'} = $p->attr('Calling-Station-Id');
			$USM{$id}->{'Called-Station-Id'} = $p->attr('Called-Station-Id'); 
			$USM{$id}->{'Acct-Input-Octets'} = $p->attr('Acct-Input-Octets');
			$USM{$id}->{'Acct-Output-Octets'} = $p->attr('Acct-Output-Octets');
			$USM{$id}->{'NAS-Id'} = $host; 
			$USM{$id}->{'NAS-Port-Id'} = $p->attr('NAS-Port-Id');
			$USM{$id}->{'Framed-IP-Address'} = $p->attr('Framed-IP-Address');
			$USM{$id}->{'Dead-Flag'} = 0;
			$USM_IP{$USM{$id}->{'Framed-IP-Address'}} = $USM{$id};
	LOG("New session $id added to USM. Type is ".$USM{$id}->{'Service-Type'});
}

sub USM_MARKDEAD_SESSION {
	my($p)    = shift @_;
	my($id)   = shift @_;
	my($host) = shift @_;
	my($port) = shift @_;
	my($key);

	if($id) {
        	$USM{$id}->{'Dead-Flag'}++; ### Mard session DEAD!
 
	       # Now, trying to associate the Flow with USM...

		foreach $key (keys %{ $USM_FLOW{$USM{$id}->{'Framed-IP-Address'}} }) {
			FLOW_DUMP_LINE($key) if($FLOW{$key} >= $FLOW_REDUCE_BYTES);
		}

	        #foreach $key (keys %FLOW) {
		#	($_, $_, $ip_dst) = split(' ',$key);
        	#        if($USM{$id}->{'Framed-IP-Address'} eq $ip_dst) {
		#		FLOW_DUMP_LINE($key);
                #	}
        	#}

		USM_UPDATE_SESSION($p, $id, $host, $port, 'finish');

		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($USM{$id}->{'Acct-Session-Time'});
                $mon++;   
                $year += 1900;

		### delete $USM{$id};
		LOG("Session $id marked dead.");

	} else {
		LOG("Cannot mark session dead, weird id: $id");
	}
}

sub USM_REMOVE_SESSION {
	my($p)    = shift @_;
	my($id)   = shift @_;
	my($host) = shift @_;
	my($port) = shift @_;
	my($key);

	if($id) {
 
	       # Now, trying to associate the Flow with USM...


	        #foreach $key (keys %FLOW) {
		#	($_, $ip_src, $ip_dst) = split(' ',$key);
        	#        if($USM{$id}->{'Framed-IP-Address'} eq $ip_dst || $USM{$id}->{'Framed-IP-Address'} eq $ip_src) {
		#		FLOW_DUMP_LINE($key);
                #	}
        	#}

		foreach $key (keys %{ $USM_FLOW{$USM{$id}->{'Framed-IP-Address'}} }) {
			FLOW_DUMP_LINE($key) if($FLOW{$key} >= $FLOW_REDUCE_BYTES);
		}

		### USM_UPDATE_SESSION($p, $id, $host, $port, 'finish');

		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($USM{$id}->{'Acct-Session-Time'});
                $mon++;   
                $year += 1900;


		delete $USM_IP{$USM{$id}->{'Framed-IP-Address'}};
		delete $USM_FLOW{$USM{$id}->{'Framed-IP-Address'}};
		delete $USM{$id};
		LOG("Session $id removed.");

	} else {
		LOG("Cannot remove session, weird id: $id");
	}
}

sub USM_UPDATE_SESSION {
	my($p)    = shift @_;
	my($id)   = shift @_;
	my($host) = shift @_;
	my($port) = shift @_;
	my($cmd) = shift @_;
	my($cur_time) = time;

	# Do accounting first...

	# this is our own opinion on what session time was... 
	my($long) = $cur_time - $USM{$id}->{'Acct-Session-Time'}; #

	# if CISCO sends us its opinion on what session time was, use it!!!	
	# if( $p && defined($p->attr('Acct-Session-Time')) ) {
		###	printf( "--- my=%d, his=%d\n\n", 
		###	$long, $p->attr('Acct-Session-Time'));
	#	$long = $p->attr('Acct-Session-Time');
	#}

        # yet another CISCO bug: voip_ip + alive
        if( $p && defined($p->attr('User-Name') && $USM{$id}->{'Service-Type'} =~ /voip_ip/) ) {
                # use user-name from Alive messages. this is a work-around for CISCO bug!!!
                $USM{$id}->{'User-Name'} = $p->attr('User-Name');
        }

	#
        if( $p && defined($p->attr('Framed-IP-Address') ) ) {
                $USM{$id}->{'Framed-IP-Address'} = $p->attr('Framed-IP-Address');
        }

	SEND_TO_TARIF_SERVER(
		$cmd,				# CMD
		$id.':'.$USM{$id}->{'Acct-Session-Time'}, # "$sid@$host:time" 
		$USM{$id}->{'Service-Type'},		# service type (voip_ip ... )
		$USM{$id}->{'Acct-Session-Time'},	# session begin time
		$long,				# session long 
		$USM{$id}->{'User-Name'},  	# user's id
		"", 				# password 
		$host,				# NAS ID
		$USM{$id}->{'Calling-Station-Id'}, 	# src major address
		$USM{$id}->{'Calling-Port-Id'}, 	# src minor address
		$USM{$id}->{'Called-Station-Id'}, 	# dst major address
		$USM{$id}->{'Called-Port-Id'}, 		# dst minor address 
		$USM{$id}->{'Acct-Input-Octets'},	# bytes in 
		$USM{$id}->{'Acct-Output-Octets'},	# bytes out 
		REMOTE_MEDIA_ADDRESS($p),
		DISCONNECT_CAUSE($p),
		VOICE_QUALITY($p)
	);

	# Update USM entry...

	if( $p ) {
	$USM{$id}->{'Acct-Session-Alive'} = $cur_time - $p->attr('Acct-Delay-Time');
	$USM{$id}->{'Calling-Station-Id'} = $p->attr('Calling-Station-Id');
	$USM{$id}->{'Called-Station-Id'} = $p->attr('Called-Station-Id'); 
	$USM{$id}->{'Acct-Input-Octets'} = $p->attr('Acct-Input-Octets');
	$USM{$id}->{'Acct-Output-Octets'} = $p->attr('Acct-Output-Octets');
	} else {
	$USM{$id}->{'Acct-Session-Alive'} = $cur_time;
	}

	LOG("Session $id updated.");
}


sub CISCO_NAS_PORT {
	my($p) = shift @_;

			foreach (
				values %{$p->{Attributes}} ) {
					#LOG("CISCO_NAS_PORT: $_");
			}
				#grep /cisco-nas-port/, 
}

sub SERVICE_TYPE {
	my($p) = shift @_;

	if( $p->attr('Framed-Protocol') eq 'PPP' ||  
	    $p->attr('Framed-Protocol') eq 'SLIP' || 
	    $p->attr('Framed-Protocol') eq 'PPPOE') {
			return 'inet_dynamic_ip';
	} elsif ( $p->attr('Service-Type') eq 'Call-Check' ) {
			return 'voip_gk';
	} elsif ( $p->attr('Service-Type') eq 'Login-User' ||
		  $p->attr('Service-Type') eq 'Voice' ) {

			#foreach (
			#	values %{$p->{Attributes}} ) {
			#		LOG("SERVICE_TYPE: $_");
			#}
					

			my $voip_card = 0;
			my $ani_phone = 0;
			my $pots_phone = 0;
			my $voip_ip = 0;
			my $pay_card = 0;
			my $pay_card_payment = 0;
			my $telephone_info = 0;
			my $voip_gk = 0;
			my $web_call = 0;
			my $h323 = 0;

			foreach (
				grep /h323/, 
				values %{$p->{Attributes}} ) {
					LOG("H323: $_");
					$h323 ++;
					if(/telephone_info:true/) 
						{ $telephone_info ++; next; }
					if(/web_call:true/) 
						{ $web_call ++; next; }
					if(/debitcard:true/) 
						{ $voip_card ++; next; }
					if(/paycard:true/) 
						{ $pay_card ++; next; }
					if(/paycard_payment:true/) 
						{ $pay_card_payment ++; next; }
					if(/remote_ip:true/) 
						{ $voip_ip ++; next; }
					if(/aniphone:true/) 
						{ $ani_phone ++; next; }
					if(/type=Telephony/) # VoIP -> ISDN
						{ $voip_ip ++; next; }
					if(/terminal-alias/) # GNUGK 2.0.8 
						{ $voip_gk ++; next; }
					if(/h323-call-origin=proxy/) # GNUGK 2.0.8 !!! THIS COMPROMISE WITH pots_phone 
						{ $voip_gk ++; next; }
			} 

			if($h323 && $p->attr('Framed-IP-Address')) 
				{ return 'voip_gk'; }

			if(!$web_call && !$voip_ip && !$telephone_info && !$pay_card_payment && !$pay_card && !$voip_card && !$ani_phone && ($p->attr('Calling-Station-Id') =~ /h323_ID/ ||
			   $p->attr('Calling-Station-Id') =~ /dialedDigit/ || 
				( $p->attr('Calling-Station-Id') eq '' && $p->attr('Called-Station-Id') eq '')))
				{ return 'voip_gk'; }


			if($web_call) { return 'web_call'; }
			if($telephone_info) { return 'telephone_info'; }
			if($pay_card_payment) { return 'pay_card_payment'; }
			if($pay_card) { return 'pay_card'; }
			if($voip_card) { return 'voip_card'; }
			if($ani_phone) { return 'ani_phone'; }
			if($voip_ip) { return 'voip_ip'; }	
			if($voip_gk) { return 'voip_gk'; }	

			return 'pots_phone'; ### Use this by default
	} else {
		return 'unknown';
	}
}


sub SEND_TO_TARIF_SERVER {

	my $TARIFF_SOCK;
	my $TRQ = "";
	my ($ts) = Time::HiRes::time;


	foreach (@_) { $TRQ = $TRQ.$_."\t"; }
	$TRQ = $TRQ . "\n";




	if($TRQ =~ /^auth/) {
		# rotate sockets

		$TARIFF_SOCK = shift @tariff_sockets_for_auth; 
		push @tariff_sockets_for_auth, $TARIFF_SOCK if($TARIFF_SOCK);

		if(! $TARIFF_SOCK) {
			LOG("No prioritized tariffication servers available, request skipped");
			return -1;
		}

		$sw = syswrite($TARIFF_SOCK, $TRQ, length($TRQ));
		LOG("TRQ sent to auth: $TARIFF_SOCK : syswrite = $sw");

		if($sw < 0 || !defined($sw)) {
			SOCKET_REMOVE_FROM_AUTH($TARIFF_SOCK);
			return -1;
		}
		if($sw < 1) {
			return -1
		}


	} else {
		# rotate sockets

		$TARIFF_SOCK = shift @tariff_sockets; 
		push @tariff_sockets, $TARIFF_SOCK if($TARIFF_SOCK);

		if(! $TARIFF_SOCK) {
			LOG("No tariffication servers available, request skipped");
			return -1;
		}

		$sw = syswrite($TARIFF_SOCK, $TRQ, length($TRQ));
		LOG("TRQ sent to: $TARIFF_SOCK : syswrite = $sw");

		if($sw < 0 || !defined($sw)) {
			SOCKET_REMOVE($TARIFF_SOCK);
			return -1;
		}
		if($sw < 1) {
			return -1
		}

	}
	LOG("TRQ delay = ".sprintf("%00.03f",Time::HiRes::time - $ts));
}


sub RSM_ADD {
	my($p)    = shift @_;
	my($host) = shift @_;
	my($port) = shift @_;

	### my($id)     = $p->identifier.":".$p->authenticator.'@'.$host;
	my($id)     = $p->identifier.'@'.$host;

### REMOVE CODE BELOW ###
	$host =~ s/^hack_//;
### REMOVE CODE ABOVE ###

	if( defined($RSM{$id}) ) {
		LOG("RSM session $id already exists");
### FIX ME: return 0 - skip duplicate request, 1 - proceed with it
		return 1;
	} else {
		$RSM{$id}->{'Host'} = $host;
		$RSM{$id}->{'Port'} = $port;
		$RSM{$id}->{'Start'} = time;
		$RSM{$id}->{'Service-Type'} = SERVICE_TYPE($p);
		$RSM{$id}->{'Request'} = $p;

		$stat_rsms += 1;
		LOG("New request $id added to RSM");
		return 1;
	}
}

sub RSM_DISPLAY {

	$debug++;
	if($debug == 6) { $debug = 0; }
	print STDERR "\n\n--- New DEBUG level is: $debug\n";

	foreach (keys %RSM) {
		LOG(sprintf("RSM: id = %s, start = %d, host = %s, port = %d, type = %d", $_, $RSM{$_}->{'Start'}, $RSM{$_}->{'Host'}, $RSM{$_}->{'Port'}, $RSM{$_}->{'Request'}->code ));
	}

}

sub RSM_REMOVE {
	my($id)  = shift @_;

	delete $RSM{$id};
	$stat_rsms -= 1;
	LOG("Request $id removed from RSM");
}

sub REMOTE_MEDIA_ADDRESS {
	my($p) = shift @_;
	$_ = "";
	foreach ( grep /remote-media-address/, values %{$p->{Attributes}} ) {	
		s/.*remote-media-address=//g;
		return $_;
	}
}

sub DISCONNECT_CAUSE {
	my($p) = shift @_;
	$_ = "";
	foreach ( grep /disconnect-cause/, values %{$p->{Attributes}} ) {	
		s/.*disconnect-cause=//g;
		return $_;
	}
}

sub VOICE_QUALITY {
	my($p) = shift @_;
	$_ = "";
	foreach ( grep /voice-quality/, values %{$p->{Attributes}} ) {	
		s/.*voice-quality=//g;
		return $_;
	}
}

sub process_netflow {
	my($SOCK)   = shift @_;
	my($data) = shift @_;
	my($his)    = shift @_;

	$stat_netflow_requests_received += 1;

	($port, $hisiaddr) = sockaddr_in($his);
	#$host = gethostbyaddr($hisiaddr, AF_INET);
	$host = inet_ntoa($hisiaddr);
	my($cur_time)  = time;
	my($secret)    = $radius_client{$host};
	my($i);
	

	LOG("*** His addr: $host, port: $port") if($debug_level > 0);

	if(! $secret) {
           LOG("Host is not listed as a client, skip request");	
	   return;
	}


	($ver, $count, $uptime, $unix_secs, $unix_nsecs,
		$flow_seq, $data ) = unpack("n n N N N N a*", $data);

        LOG("Netflow packet info: ver=$ver, count=$count, seq=$flow_seq");

	if( $ver == 5) {
		($reserved, $data) = unpack ("a4 a*", $data);
		for($i = 0; $i < $count; $i++) {
			($src_ip_addr, $dst_ip_addr, $next_hop, $mib_in,
			 $mib_out, $pkts, $bytes, $u_start, $u_last,
			 $src_port, $dst_port, $flags, $src_as, $dst_as, 
			 $_, $data)
			= unpack("a4 a4 a4 n n N N N N n n N n n a4 a*", $data);

			$src_ip_addr = inet_ntoa($src_ip_addr);
			$dst_ip_addr = inet_ntoa($dst_ip_addr);
			$next_hop = inet_ntoa($next_hop);

LOG("+++ $src_ip_addr:$src_port $dst_ip_addr:$dst_port $next_hop $mib_in $mib_out $pkts $bytes $u_start:$u_last $src_as $dst_as") if($debug_level > 1);

			 next if( $bytes < $FLOW_REDUCE_BYTES_INPUT);
                                
			 if(exists $USM_IP{$src_ip_addr}) {
				$USM_FLOW{$src_ip_addr}{$host." ".$src_ip_addr." ".$dst_ip_addr}++;
			 }

			 if(exists $USM_IP{$dst_ip_addr}) {
				$USM_FLOW{$dst_ip_addr}{$host." ".$src_ip_addr." ".$dst_ip_addr}++;
			 }

			 $FLOW{$host." ".$src_ip_addr." ".$dst_ip_addr} += 
				$bytes;
			 $FLOW_TIME{$host." ".$src_ip_addr." ".$dst_ip_addr} = 
				time;
			 FLOW_DUMP_LINE($host." ".$src_ip_addr." ".$dst_ip_addr)
				if($FLOW{$host." ".$src_ip_addr." ".$dst_ip_addr} >= $FLOW_TRIGGER_BYTES_UNASSOCIATED);
                        }
	} else {
		LOG("Netflow packet version $ver is not supported!");
	}
}


sub FLOW_DUMP_LINE {

	my($key) = shift @_;
	my($bytes, $host, $ip_src, $ip_dst);
	my($direction) = 0;

	$bytes = $FLOW{$key};
	($host, $ip_src, $ip_dst) = split(' ',$key);

	my($user_id, $session_id) = 
			('', "netflow-".rand(1000000)."\@".$host.":".$FLOW_TIME{$key});

	my($service_type) = "inet_fixed_ip"; ### Use it by default!


	# Now, trying to associate the Flow with USM...

	foreach $id (keys %USM) {

		if($USM{$id}->{'Framed-IP-Address'} eq $ip_src) {
			$user_id = $USM{$id}->{'User-Name'};
			$service_type = $USM{$id}->{'Service-Type'};
			$session_id = $id.":".$USM{$id}->{'Acct-Session-Time'};
			$direction = 2; # FROM CLIENT
			last;
		}
		if($USM{$id}->{'Framed-IP-Address'} eq $ip_dst) {
			$user_id = $USM{$id}->{'User-Name'};
			$service_type = $USM{$id}->{'Service-Type'};
			$session_id = $id.":".$USM{$id}->{'Acct-Session-Time'};
			$direction = 1; # TO CLIENT
			last;
		}
	}

	if(SEND_TO_TARIF_SERVER(
		'netflow',		# CMD
		$session_id,	# fake session for netflow  
		$service_type,		# service type (voip_ip ... )
		### $FLOW_TIME{$key},	# session begin time
		time,			# session begin time
		'0',			# session long 
		$user_id,  		# user's id
		"", 			# password 
		$host,			# NAS ID
		$ip_src, 		# src major address
		'', 			# src minor address
		$ip_dst, 		# dst major address
		'', 			# dst minor address 
		0,			# bytes in 
		$bytes,			# bytes out 
		,			# ???
		,			# ???
		,			# ???
		$direction,		# TO CLIENT/FROM CLIENT	
	) == 0) {
		LOG "\n***	$host	$user_id	$session_id	$ip_src		$ip_dst		$bytes\n" 
			if($debug_level > 2);
		delete $FLOW{$key};
		delete $FLOW_TIME{$key};
		$stat_netflow_dumps += 1;
	} else {
		LOG("TRQ error");
	}
}

sub FLOW_CHECK {

	my($tstamp) = time;
	my($count_checked, $count_removed, $count_dumped)  = (0, 0, 0);
	my($id,$associated,$ip_host,$ip_src,$ip_dst);
	
	if($tstamp - $flow_last_check >= $FLOW_CHECK_INTERVAL) {
		LOG("NETFLOW: checking started...");
		my($ts) = Time::HiRes::time;
		my($total_associated) = 0;
		my($total_unassociated) = 0;
		my($last_dumped_key);
		my($last_dumped_bytes);

		foreach (keys %FLOW) {
			$count_checked += 1;

			# Check whether it is associated flow ?

			$associated = 0;
			($ip_host,$ip_src,$ip_dst) = split(' ',$_);

			$associated++ if(exists $USM_IP{$ip_src} || exists $USM_IP{$ip_dst});

			#foreach $id (keys %USM) {
                	#	if($USM{$id}->{'Framed-IP-Address'} eq $ip_src
			#	|| $USM{$id}->{'Framed-IP-Address'} eq $ip_dst)
			#		 { $associated++; last; }
			#}


			if($associated && 
			   $tstamp - $FLOW_TIME{$_} >= 
						$FLOW_EXPIRE_TIME_ASSOCIATED) { 

				if($FLOW{$_} < $FLOW_REDUCE_BYTES ) {
					delete $FLOW{$_};
					delete $FLOW_TIME{$_};
					$count_removed += 1;
				} else {
					$last_dumped_key = $_;
					$last_dumped_bytes = $FLOW{$_};
					FLOW_DUMP_LINE($_);	
					$count_dumped++;
				}
				$total_associated++;
			
			} elsif (!$associated &&
			          $tstamp - $FLOW_TIME{$_} >= 
						$FLOW_EXPIRE_TIME_UNASSOCIATED){ 
				if($FLOW{$_} < $FLOW_REDUCE_BYTES ) {
					delete $FLOW{$_};
					delete $FLOW_TIME{$_};
					$count_removed += 1;
				} else {
					$last_dumped_bytes = $FLOW{$_};
					$last_dumped_key = $_;
					FLOW_DUMP_LINE($_);	
					$count_dumped++;
				}
				$total_unassociated++;

			}
		}

		$flow_last_check = $tstamp;
		$ts = Time::HiRes::time - $ts;
		LOG("NETFLOW: checking finished, total = $count_checked, removed = $count_removed, time = $ts, associated = $total_associated, unassociated = $total_unassociated, dumped = $count_dumped");
		LOG("NETFLOW: cheking: last_dumped_key: $last_dumped_key $last_dumped_bytes");
		FLOW_SAVE();
	}
}

sub DISCONNECT {
	my($id) = shift @_;
	
	return if(! ($USM{$id}->{'Service-Type'} eq 'inet_dynamic_ip' ));
	
	if(-f $disconnect_tool{$USM{$id}->{'NAS-Id'}} ){

		$cmd = $disconnect_tool{$USM{$id}->{'NAS-Id'}}." ".
			$id." ".
			$USM{$id}->{'NAS-Id'}." ".
			$USM{$id}->{'NAS-Port-Id'}." ".
			$USM{$id}->{'Framed-IP-Address'}." ".
			$USM{$id}->{'User-Name'};

		system("$cmd &");

		if($?) {
			LOG("ERROR spawning $cmd");
		} else {
			LOG("Spawning: $cmd");
		}
	} else {
		LOG("Tool not found:  $disconnect_tool{$USM{$id}->{'NAS-Id'}}");
	}
}

sub NUMBER_OF_SESSIONS {
	my($user_name, $service_type) = @_;
	my($num_of_sessions) = 0;
	my($id);

	foreach $id (keys %USM) {

		if($USM{$id}->{'User-Name'} eq $user_name && 
		   $USM{$id}->{'Service-Type'} eq $service_type) {$num_of_sessions++;}
	} 

	return $num_of_sessions;

}

sub hex_dump_print {

        my($rest) = shift@_;
        my($char);
        my($res);

        while($rest) {
                ($char, $rest) = unpack("C a*", $rest);
                $res .= sprintf ("%02X ",$char);
        }

        return $res;
}

sub hex_dump {

        my($rest) = shift @_;
        my($char);
        my($res);

        while($rest) {
                ($char, $rest) = unpack("C a*", $rest);
                $res .= sprintf ("%02X",$char);
        }

        return $res;
}

sub hex_decode {

	my($rest) = shift @_;
	
	$rest =~ s/(..)/0x$1, /sg;
        eval " \$rest = sprintf(\"%c\"x42,$rest); ";
	return $rest;

}
