#!/usr/bin/perl


use strict;
no strict "refs";

use DBI;
use Data::Dumper;

use FileHandle;
use Time::HiRes;

require "/etc/cherry/cherry.cfg";
require "/usr/lib/cherry/tariffsrv/common.pl";
require "/usr/lib/cherry/dateutils";

my %services;
sub reload; 
sub do_cmd; 
sub get_service;

$main::DEBUG = 1 ; 

##########################################################################
sub LOG {
        my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
        my($date) = sprintf "%02d.%02d.%04d %02d:%02d:%02d", $mday, $mon+1, $year+1900, $hour, $min, $sec;


        foreach my $logstr (@_) {
                #$main::LOG_DATA .= "$$\t$date\t$logstr\n";

		print "$$\t$date\t$logstr\n";
        }

}
##########################################################################
sub get_service {
	my ($oper_id,$user_id,$session_begin) =@_;

	foreach my $srv ( sort{$b->{'date_create'} cmp $a->{'date_create'} } 
				@{ $services{$oper_id}{$user_id} }) 
	{




        	if ($session_begin ge $srv->{'date_create'}) {
			if (! defined($srv->{'date_expire'}) ) {
				return $srv;
			} else {
				if( $session_begin le $srv->{'date_expire'} ) {
					return $srv;
				}
                	}
		}

        }

	return;
}



###################################################################################

sub reload {
	my ($dbh,$service) = @_;

	my $sth;
	my $rv;
	my $sql;


        ##### load services ###########################

        $sql = 	"SELECT  oper_id, ".
		" client_id, ".
		" user_id, ".
		" enabled, ".
		" date_create, ".
		" date_active, ".
		" date_expire, ".
		" tariff_plan, ".
		" water_mark, ".
		" password, ".
		" password_type ".
		" FROM ${service}_services ";

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);
        $sth = $dbh->prepare($sql);
        if( !$sth ) {
                main::LOG ("ERROR: SQL request syntax error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
                main::LOG ("ERROR: SQL request execution error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        while( my ( $oper_id, $client_id, $user_id, $enabled,$date_create, 
		$date_active,$date_expire,$tariff_plan,$water_mark,$password,$password_type,
		) = $sth->fetchrow_array)
	{
		push @{ $services{$oper_id}{$user_id} },
			{
				oper_id => $oper_id,
				client_id => $client_id,
				user_id => $user_id, 
				enabled => $enabled,
				date_create => $date_create,
				date_active => $date_active,
				date_expire => $date_expire,
				tariff_plan => $tariff_plan,
				water_mark => $water_mark,
				password => $password,
				password_type => $password_type,
			};
	};


	return 'OK';

}

##################################################################################
sub do_cmd { 


	my (
		$dbh,
                $cmd,
                $query_id,
                $service_type,
                $session_begin,
                $session_long,
                $user_id,
		$password,
                $nas_id,
                $src_major,
		$src_minor,
                $dst_major,
                $price,


	) = @_;


	
        my ($oper_id,$nas_type) = &common::get_oper_id($nas_id);
        if(!defined($oper_id)) {
                main::LOG ("ERROR: oper_id not found (nas_id=$nas_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\toper_id not found (nas_id=$nas_id)\n";
        }



        my $srv=get_service($oper_id,$user_id,$session_begin);
        if(!defined($srv)) {
        	main::LOG ( "ERROR: user_id not found " .
                	"(oper_id=$oper_id, user_id=$user_id, session_begin=$session_begin)") 
			if ($main::DEBUG>0);

                return "ERROR\t$query_id\tuser_id not found" .
                	"(oper_id=$oper_id, user_id=$user_id, session_begin=$session_begin)"."\n";

        }

	my $client_id = $srv->{'client_id'};
        if(!defined($client_id)) {
                main::LOG ("ERROR: client_id not found (oper_id=$oper_id, user_id=$user_id)"
			) if ($main::DEBUG>0);

                return "ERROR\t$query_id\tclient_id not found (oper_id=$oper_id, " .
			"user_id=$user_id)\n";
        }


        my $water_mark = 1 * $srv->{'water_mark'};

        my ($balance,$balance_currency_type,$options,$client_type) =
                        &common::get_client_balance($dbh,$oper_id,$client_id);

        if(!defined($balance) ) {
                main::LOG ("ERROR: can't get balance (oper_id=$oper_id, " .
                "client_id=$client_id)") if ($main::DEBUG>0);

                return "ERROR\t$query_id\tcan't get balance (oper_id=$oper_id, " .
                        "client_id=$client_id)\n";
        }

        if(!defined($balance_currency_type) ) {
                main::LOG ("ERROR: balance's currency_type not found ".
                "(oper_id=$oper_id, client_id=$client_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\tbalance's currency_type not found ".
                        "(oper_id=$oper_id, client_id=$client_id)\n";
        }


        my $ret=&common::save_session (
                $dbh,
                $query_id,
                $service_type,
                $session_begin,
                $session_long,
                $user_id,
                $src_major,
                "",
                $dst_major,
                "",
                0, #$bytes_in,
                0, #$bytes_out,
                $nas_id,
                "",
                "",
                "",
                );

                if($ret!~ /OK/) {
                        main::LOG ("ERROR: can't save session") if ($main::DEBUG>0);
                        return "ERROR\t$query_id\t can't save session \n";

                }




	my $status=7;
        &common::save_session_status ($dbh, $query_id, $status);

	my $tariff=0;
	$tariff = $price / ($session_long / 60 ) if($session_long>0);
	my $tariff_id=444;

        $ret = &common::update_balance (
			0,
                        $dbh,
                        $query_id,
                        $session_long,
                        "sec",
                        "07",
                        "07",
                        $balance_currency_type,
                        $price,
                        $user_id,
                        $oper_id,
                        $client_id,
			"",
			"",
                        $dst_major,
                	$tariff,
                	$tariff_id,
			1,
			$service_type,
                        "",
			$session_begin,
			$session_long,
                	);

                	if(!($ret eq 'OK')) {
                        	main::LOG ("ERROR: can't update balance") if ($main::DEBUG>0);
                        	return "ERROR\t$query_id\tcan't update balance\n";
                	}



        return "OK\t$query_id\t$balance\t0\t$balance_currency_type\t0\t0\n";

}
#########################################################################


##
##
## MAIN
##
##
##


my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

if( !$dbh ) {
                LOG ("get_db_connection\tERROR: Access to database failed!") if ($main::DEBUG>0);
}


common::reload($dbh);
reload($dbh,'telephone');


while(<>) {
	my (
                	$cmd,
                	$query_id,
                	$service_type,
                	$session_begin,
                	$session_long,
                	$user_id,
			$password,
                	$nas_id,
                	$src_major,
			$src_minor,
                	$dst_major,
                	$price,
	) = split("\t",$_);

	$session_begin = common::time2str($session_begin);

	print do_cmd($dbh,
                        $cmd,
                        $query_id,
                        $service_type,
                        $session_begin,
                        $session_long,
                        $user_id,
                        $password,
                        $nas_id,
                        $src_major,
                        $src_minor,
                        $dst_major,
                        $price,
	);

}

$dbh->disconnect;


