package common;

use strict;
no strict "refs";

use Time::Local;


use Data::Dumper;


my %nas_list;
my %currency_history; 
my %currency_types;
my %units_translation;
my %holidays; 

##### Used tables #############
# base_nas_list
# base_currency_types
# base_currency_history
# base_units_translation
###############################

sub reload ;
sub utime2pgtime ;
sub time2str;
sub str2time;
sub get_client_balance ;
sub get_currency_rate ;
sub translate_currency ;
sub get_currency_gender ;
sub update_balance ;
sub save_session ;
sub get_oper_id;
sub translate_unit;
sub check_time_zone;
sub get_inet_addr;

############################################################################################
sub reload {
	my ($dbh) =@_;

        my $sth;
        my $rv;
        my $sql;

	########## load nas_list #######################################
        $sql = 	" SELECT  base_nas_list.nas_id, " .
        	" base_nas_list.oper_id, " .
		" base_nas_list.nas_type " .
                " FROM    base_nas_list ";

        main::LOG("SQL:\n$sql\n") if ($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
                main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        while( my ($nas_id,$oper_id,$nas_type) =  $sth->fetchrow_array) {
                $nas_list{$nas_id}->{'oper_id'} = $oper_id;
		$nas_list{$nas_id}->{'nas_type'} = $nas_type;
        }


	####### load currency_types #############################33

	foreach my $currency_type ( keys %{$CFG::TARIFF_CURRENCY_GENDER}) {
                $currency_types{$currency_type}->{'gender_h'} = 
			$CFG::TARIFF_CURRENCY_GENDER{$currency_type}->{dollar};
		$currency_types{$currency_type}->{'gender_l'} = 
			$CFG::TARIFF_CURRENCY_GENDER{$currency_type}->{cent};
        }


	########## load currency_history #########################################33

        $sql = 	"SELECT  base_currency_history.oper_id, " .
                " base_currency_history.currency_from, ". 
                " base_currency_history.currency_to, ".
                " base_currency_history.date, ".
		" base_currency_history.rate ".
                " FROM    base_currency_history ";

        main::LOG("SQL:\n$sql\n") if ($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        while( my ($oper_id,$currency_from,$currency_to,$date,$rate) =  $sth->fetchrow_array) {
		push @{$currency_history{$oper_id}{$currency_from}{$currency_to}}, [$date, $rate];
        }

        ########## load units_translation #########################################33

        $sql = 	"SELECT  base_units_translation.unit_from, ".
		" base_units_translation.unit_to, ".
		" base_units_translation.rate ".
                " FROM    base_units_translation ";

        main::LOG("SQL:\n$sql\n") if ($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        while( my ($unit_from,$unit_to,$rate) =  $sth->fetchrow_array) {
                $units_translation{$unit_from}{$unit_to}->{'rate'}=$rate;
        }

        ########## load holidays #########################################33

        $sql = " SELECT  oper_id,date_,day_type FROM   base_holidays ";

        main::LOG("SQL:\n$sql\n") if ($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
                main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
                main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return ;
        }

        while( my ($oper_id,$date_,$day_type) =  $sth->fetchrow_array) {
                $holidays{$oper_id}{$date_}={day_type=>$day_type};
        }


	return 'OK';
}

#############################################################################################
sub utime2pgtime {
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(shift @_);
        $mon++;
        $year += 1900;
	return sprintf("%.4i-%.2i-%.2i %.2i:%.2i:%.2i",$year,$mon,$mday,$hour,$min,$sec);
}
#############################################################################################
sub time2str {
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(shift @_);
        $mon++;
        $year += 1900;
        return sprintf("%.4i-%.2i-%.2i %.2i:%.2i:%.2i",$year,$mon,$mday,$hour,$min,$sec);
}
############################################################################################
sub str2time {
	my ($str) = @_;

	if($str =~ /(\d\d\d\d)\-(\d\d)\-(\d\d)\ (\d\d)\:(\d\d)\:(\d\d)/) {
        	my ($year,$mon,$mday,$hours,$min,$sec) = ($1,$2,$3,$4,$5,$6);
                return timelocal($sec,$min,$hours,$mday,$mon-1,$year-1900);
	}

	return;
}
############################################################################################
sub get_client_balance {
	#this sub return balance, currency_type use client_id

        my ($dbh,$oper_id,$client_id) = @_;

        my $sth;
        my $rv;
        my $sql;

        $sql = 	"SELECT ".
                " base_clients.balance, ".
                " base_clients.currency_type, ".
		" base_clients.options, ".
		" base_clients.client_type ".
                " FROM    base_clients ".
                " WHERE   base_clients.oper_id='$oper_id' AND base_clients.client_id = '$client_id' ";

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return;
        }

        $rv = $sth->execute;
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                return;
        }

        my ($balance,$currency_type,$options,$client_type) =  $sth->fetchrow_array;
        return ($balance,$currency_type,$options,$client_type);
} 	
#############################################################################################
sub get_currency_rate {
        my ( $oper_id,$currency_type_from,$currency_type_to,$time_begin) = @_;

        my $rates = $currency_history{$oper_id}{$currency_type_from}{$currency_type_to};

        if(!defined($rates)) {
                return;
        }

        my ($date, $rate);
        foreach (sort {$b->[0] cmp $a->[0]} @$rates) {
                ($date, $rate) = @$_;
                if($time_begin ge $date) {
                        last;
                }
        }

        return ($rate);
} 
#############################################################################################
sub translate_currency {
        my ( $oper_id,$currency_type_from,$currency_type_to,$time_begin,$amount) = @_;

	my $rate;
	
	if($currency_type_from eq $currency_type_to) {
		$rate=1;
	} else {
		$rate=get_currency_rate($oper_id,$currency_type_from,$currency_type_to,$time_begin);
	}



	if(defined($rate)) {
		return $amount*$rate;
	}

	return;
}
#############################################################################################
sub translate_unit {
	my ($unit_from,$unit_to,$amount) = @_;
	my $rate ;


	if($unit_from eq $unit_to) {
		$rate=1;
	} else {
		$rate= $units_translation{$unit_from}{$unit_to}->{'rate'};
	}

	if(defined($rate)) {
		return $amount*$rate;
        }

        return;
}
#############################################################################################
sub get_currency_gender {
        my  ($currency_type) = @_;


	# 0 - mail; 1 -femail;

	my $gender_h=0;
	my $gender_l=1;

	if(defined($currency_types{$currency_type})) {
        	$gender_h=$currency_types{$currency_type}->{'gender_h'}; 
		$gender_l=$currency_types{$currency_type}->{'gender_l'};
	}

        return ($gender_h,$gender_l);
}
#############################################################################################

sub update_balance {
	my($res1,$res2) = update_balance2(@_);
	return $res1;
}

sub update_balance2 {
	#this sub update client's balance and update record in base_calculated_session table
	# and base_session
	my (
		$update_type, 	
		$dbh,
                $session_id,
		$resource,
        	$resource_type,
        	$direction,
        	$tariff_zone,
        	$currency_type,
        	$amount,
        	$user_id,
        	$oper_id,
        	$client_id,
                $tariff_plan,
                $tariff_sub_plan,
		$direction_descr,
		$tariff,
		$tariff_id,
		$update_direction,
		$service_type,
		$zone_plan,
		$session_begin_calc,
		$session_long_calc,

	) = @_;
		
	my $sth;
	my $rv;
	my $sql;


        $sql = 	"SELECT amount,calc_session_id FROM base_calculated_sessions " .
                        "WHERE session_id = '$session_id' AND " .
                        "service_type='$service_type' AND ".
                        "oper_id='$oper_id' AND ".
                        "tariff_id=$tariff_id ";


        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $sth = $dbh->prepare($sql);
        if( !$sth ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		$dbh->rollback;
                return ;
        }


        $rv = $sth->execute;
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		$dbh->rollback;
                return ;
        }


        my $old_amount = 0;
	my $amount_change = 0;
	my $calc_session_id=0;

        if($rv>0) {
		
		my $amount_update;
		my $resource_update;
		my $session_long_calc_update;


		($old_amount,$calc_session_id)=$sth->fetchrow_array;

		if($update_type){
			$amount_change = $amount;
			$amount_update = "amount = amount + ( ${amount_change}::numeric )";
                        $resource_update= "resource = resource  + ( ${resource}::numeric )";
			$session_long_calc_update = "session_long_calc = session_long_calc + ( $session_long_calc )";
		} else {
			$amount_change = $amount - $old_amount;
			$amount_update = "amount = amount + ( ${amount_change}::numeric ) ";
			$resource_update= "resource = $resource";
			$session_long_calc_update = "session_long_calc = $session_long_calc ";
		}

        	$sql = "UPDATE base_calculated_sessions SET ";

			if($update_direction) {
			$sql .= " direction='$direction', tariff_zone='$tariff_zone', " .
				" direction_descr='$direction_descr', ";
			}

			$sql .= "$amount_update , " .
			"$resource_update, " .
			"$session_long_calc_update " .
               		"WHERE session_id = '$session_id' AND " .
			"service_type='$service_type' AND ".
			"oper_id='$oper_id' AND ".
			"tariff_id=$tariff_id ";

        } else {
		$amount_change = $amount;
        	($calc_session_id) =  $dbh->selectrow_array("SELECT nextval('base_calculated_sessions_calc_session_id')");

       		$sql = "INSERT INTO base_calculated_sessions " .
                	"(session_id,direction,tariff_zone,currency_type,".
			"amount,resource,resource_type,client_id, oper_id, " .
			"tariff_plan, tariff_sub_plan, direction_descr, tariff, tariff_id, " .
			"service_type,zone_plan,session_begin_calc,session_long_calc,calc_session_id) ".
                	"VALUES ".
			"('$session_id','$direction','$tariff_zone',".
			"'$currency_type',".
			"'$amount_change','$resource','$resource_type','$client_id','$oper_id'," .
			"'$tariff_plan', '$tariff_sub_plan','$direction_descr', " .
			"($tariff)::numeric,$tariff_id,'$service_type','$zone_plan', " .
			"'$session_begin_calc', $session_long_calc,$calc_session_id)";

        }

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $rv = $dbh->do( $sql );
        if( !$rv ) {
		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		$dbh->rollback;
                return ;

        }

	if($amount_change != 0 ) {
        	$sql = " UPDATE base_clients " .
		"SET balance=balance - ( ${amount_change}::numeric ) " .
		"WHERE client_id='$client_id' AND oper_id='$oper_id' ";

        	main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        	$rv = $dbh->do( $sql );
        	if( !$rv ) {
			main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
			$dbh->rollback;
                	return ;
        	}
	}

        $dbh->commit ;


        return 'OK',$calc_session_id;

}


###########################################################################################
sub save_session {
	my (
		$dbh,
		$session_id,
		$session_type,
		$session_begin,
		$session_long,
		$user_id,
		$src_addr_major,
		$src_addr_minor,
		$dst_addr_major,
		$dst_addr_minor,
		$bytes_in,
		$bytes_out,
		$nas_id,
		$via,
                $disconnect_cause,
                $voice_quality
	) = @_;


        my $sth;
        my $rv;
        my $sql;

        $sql = "UPDATE base_sessions SET session_type='$session_type', " .
	"session_begin='$session_begin', session_long='$session_long', ".
        "user_id='$user_id', src_addr_major='$src_addr_major', ".
	"src_addr_minor='$src_addr_minor', dst_addr_major='$dst_addr_major', ".
	"dst_addr_minor='$dst_addr_minor', bytes_in='$bytes_in', bytes_out='$bytes_out', ".
	"nas_id='$nas_id', via='$via', disconnect_cause='$disconnect_cause', ".
	"voice_quality='$voice_quality' WHERE  session_id= '$session_id' ";

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $rv = $dbh->do( $sql );

        if( !$rv ) {
        	main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		$dbh->rollback;
                return ;
        }


        if(!($rv > 0)  ) {
        	$sql = "INSERT INTO base_sessions " .
                "(session_id, session_type, session_begin, session_long, ".
		"user_id, src_addr_major, src_addr_minor, ".
		"dst_addr_major, dst_addr_minor, bytes_in, bytes_out, nas_id, ".
		"via, disconnect_cause,voice_quality) ".
                "VALUES ('$session_id','$session_type','$session_begin','$session_long'," .
		"'$user_id','$src_addr_major','$src_addr_minor', ".
		"'$dst_addr_major','$dst_addr_minor','$bytes_in','$bytes_out','$nas_id',".
		"'$via','$disconnect_cause','$voice_quality') ";

		main::LOG("SQL:\n$sql\n")  if($main::DEBUG>1);

		$rv = $dbh->do( $sql );
		if( !$rv ) {
			main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
			$dbh->rollback;
			return ;
		}
	}


	$dbh->commit ;

	return 'OK';
}
############################################################################################
sub save_netflow_calc {


        my (
                $dbh,
                $netflow_session_id,
		$calc_session_id,
        ) = @_;


        my $sth;
        my $rv;
        my $sql;


        my $sth;
        my $rv;
        my $sql;


        #$sql =  "UPDATE netflow_calculated_sessions SET " .
	#	"netflow_session_id='$netflow_session_id' " .
	#	"WHERE netflow_session_id='$netflow_session_id' AND ".
	#	"calc_base_session_id=$calc_session_id ";

        #main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        #$rv = $dbh->do( $sql );

        #if( !$rv ) {
        #        main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
	#	$dbh->rollback;
        #        return ;
        #}

        #if(!($rv > 0)  ) {

 		my $sql = "INSERT INTO netflow_calculated_sessions " .
                "(netflow_session_id,calc_base_session_id) " .
                "VALUES ('$netflow_session_id', $calc_session_id) ";

        	main::LOG("SQL:\n$sql\n")  if($main::DEBUG>1);

		my $rv = $dbh->do( $sql );
		#if( !$rv ) {
       		#	main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		#	$dbh->rollback;
        	#	return ;
		#}
	#}

	$dbh->commit ;

	return 'OK';
}
############################################################################################
sub save_session_state {

	return 'OK';

        my (
                $dbh,
                $session_id,
		$state
        ) = @_;


	return 'OK';

        my $sth;
        my $rv;
        my $sql;


	$sql =  "UPDATE base_session_states SET state=$state WHERE session_id='$session_id'";
        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $rv = $dbh->do( $sql );

        if( !$rv ) {
                main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
		$dbh->rollback;
                return ;
        }

        if(!($rv > 0)  ) {

        	$sql =  "INSERT INTO base_session_states (session_id,state)".
        	"VALUES ('$session_id',0)";

        	main::LOG("SQL:\n$sql\n")  if($main::DEBUG>1);

        	$rv = $dbh->do( $sql );
        	if( !$rv ) {
        		main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
			$dbh->rollback;
                	return ;
        	}
	}

        $dbh->commit ;

        return 'OK';


}
############################################################################################

sub get_oper_id {
	my ($nas_id) = @_;
	return ($nas_list{$nas_id}->{'oper_id'}, $nas_list{$nas_id}->{'nas_type'});
}
############################################################################################
sub get_date_from_str {
        my ($session_begin) = @_;
        if($session_begin =~ /(\d\d\d\d\-\d\d\-\d\d)/) {
                return $1;
        }
        return;
}
############################################################################################

sub get_time_from_str {
        my ($session_begin) = @_;
	if($session_begin =~ /(\d\d\:\d\d\:\d\d)/) {
		return $1;
	}
	return;
}
############################################################################################


sub check_time_zone {
        my ($session_begin, $time_begin, $time_end) = @_; #

        if($session_begin =~ /(\d\d\:\d\d\:\d\d)/) {
                my $time=$1;

                return  1 * (
                ($time ge $time_begin && $time le $time_end  && $time_begin lt $time_end) ||
                ($time le $time_end && $time_begin gt $time_end) ||
                ($time ge $time_begin && $time_begin gt $time_end) ) ;
        }

}


##############################################################################################
sub type_of_day {
	my ($oper_id,$date_) = @_;

	my $wday;

	if($date_ =~ /(\d\d\d\d)\-(\d\d)\-(\d\d)\ (\d\d)\:(\d\d)\:(\d\d)/ ) {
		my($sec,$min,$hour,$mday,$mon,$year)=($6,$5,$4,$3,$2-1,$1 - 1900);
		my $unixtime=timelocal($sec,$min,$hour,$mday,$mon,$year);
		my ($isdst,$yday);
     		($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($unixtime);
	}

	my $day_type="work";
	if($wday==0 || $wday==6) {
		$day_type="holiday";
	}

	if(exists $holidays{$oper_id}{$date_}) {
		$day_type=$holidays{$oper_id}{$date_}->{holiday};
	}

	return $day_type ;
}

#########################################################################################
sub get_inet_addr {
        my ($direction) = @_;

        my($addr,$mask) = split('/',$direction);

        if( length($mask) ==0 ) {
                $mask = 32;
        }

        if( $mask>32 ) {
                main::LOG ("Invalid mask: $mask ($direction)") if($main::DEBUG>0);
                return;
        }


	my ($o1, $o2, $o3, $o4) = split('\.',$addr);

	my $iaddr = ($o1 * 256 * 256 * 256) + ($o2 * 256 * 256) + ($o3 *256) + $o4;


        $iaddr = $iaddr & (~(2**(32-$mask)-1));
        return ($iaddr,$mask);
}
###########################################################################################

sub get_counter {


	my ($dbh, $oper_id,$counter_name) = @_;

        #main::LOG("AAA get counter( $oper_id,$counter_name)");

	my $whith_counter=1;

	if($whith_counter) {

		my $sql;
		my $sth;
		my $rv;

		$sql="SELECT value FROM base_counters " .
		" WHERE oper_id='$oper_id' AND name='$counter_name'";

        	main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);
        	$sth = $dbh->prepare($sql);
        	if( !$sth ) {
               		main::LOG ("ERROR: SQL request syntax error: $! $@") if ($main::DEBUG>0);
               		return ;
        	}

        	$rv = $sth->execute;
        	if( !$rv ) {
        	main::LOG ("ERROR: SQL request execution error: $! $@") if ($main::DEBUG>0);
                return ;
        	}


        	my $counter1=$sth->fetchrow_array;
		$counter1*=1;


		return $counter1;


	}

	return;

}


sub save_counter {


        my ($dbh, $oper_id,$counter_name, $value) = @_;

	#main::LOG("AAA save counter( $oper_id,$counter_name, $value)");
        my $whith_counter=1;

        if($whith_counter) {
                my $sql;
                my $sth;
                my $rv;

                $sql="UPDATE base_counters SET value=value+($value)::numeric WHERE " .
                " oper_id='$oper_id' AND name='$counter_name'";

                main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        	$rv = $dbh->do( $sql );

        	if( !$rv ) {
                	main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                	$dbh->rollback;
                	return ;
        	}

        	if(!($rv > 0)  ) {

			$sql=" INSERT INTO base_counters " .
			" (oper_id,name,value) VALUES " .
			" ('$oper_id','$counter_name', $value ) ";

                	main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

			$rv = $dbh->do( $sql );



			if( !$rv ) {
                        main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
			$dbh->rollback;
                        return ;
                	}


                }


		$dbh->commit;


        }

        return 'OK';


}


sub  save_session_status { 

	my ($dbh, $query_id, $status)=@_;

        my $sql;
        my $sth;
        my $rv;

	$status=-999 if(!$status);

        $sql="UPDATE base_sessions SET status=$status WHERE session_id='$query_id'";
        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);

        $rv = $dbh->do( $sql );

        if( !$rv ) {
        	main::LOG (sprintf("SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                $dbh->rollback;
                return ;
        }

	$dbh->commit;
	return 'OK';

} 



1;
