#!/usr/bin/perl
use strict;

use DBI;
use Time::Local;

require "/etc/cherry/cherry.cfg";

my $logfile = "/var/log/cherry/fixed_calc_inet_fixed_ip.log";


my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

if( !$dbh ) {
	LOG ("ERROR: Access to database failed!");
	exit;
}


my $tcurrent = time();
my $tcurrent_str = time2str($tcurrent);

my $sql="SELECT user_id FROM inet_fixed_ip_services WHERE oper_id='RC' AND enabled=1 AND (date_expire IS NULL OR date_expire<'$tcurrent_str') ";

LOG("SQL:\n$sql\n");

my $sth = $dbh->prepare($sql);
if( !$sth ) {
	LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        exit -1;
}


my $rv = $sth->execute;
if( !$rv ) {
	LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        exit -1;
}


if(!(open TARIFF_CLIENT, "| /usr/lib/cherry/tariffsrv/tariff_client.pl") ) {
	LOG("Can't open tariff_client");
	exit;
}

while ( my ($user_id) = $sth->fetchrow_array ) {
	my $cmd="fixedcalc";                   	
        my $session_id="fixedcalc:$user_id:$tcurrent\@billing"; 
        my $service_type="inet_fixed_ip";     
        my $session_begin=$tcurrent;		        
        my $session_long=1;	         	
        my $host="billing";	               

	my $req = "$cmd\t$session_id\t$service_type\t$session_begin\t$session_long\t$user_id\t\t$host\n"; 
	LOG ("REQ: $req");

	if(!(print TARIFF_CLIENT $req)) {
		LOG("Can't send request to tariff_client.");
		exit;
	}
}



close (TARIFF_CLIENT);
$dbh->disconnect;
exit;

##################################################################################

sub LOG {
        my ($msg) = @_;
        my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                localtime(time);

        my($date) = sprintf "%02d.%02d.%04d %02d:%02d:%02d", $mday, $mon+1, $year+1900, $hour, $min, $sec;

        $msg = $date."\t".$msg;

        open(OUT,">>$logfile");
        print OUT $msg."\n";
        print $msg."\n";
        close(OUT);
}

#################################################################################
sub time2str {
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(shift @_);
        $mon++;
        $year += 1900;
        return sprintf("%.4i-%.2i-%.2i %.2i:%.2i:%.2i",$year,$mon,$mday,$hour,$min,$sec);
}

