#!/usr/bin/perl

package mschap2;

use vars qw(@ISA @EXPORT $VERSION);
@ISA = qw(Exporter);
$VERSION = '0.1';
@EXPORT = qw( CheckMSCHAP2 EncodeMSCHAP2 DecodeMSCHAP2 GenerateNTResponse GenerateAuthenticatorResponse );


use Crypt::SmbHash qw(ntlmgen lmhash nthash mdfour);
use Digest::SHA1;
use Crypt::DES;
use strict;

sub CheckMSCHAP2 {
	my ($UserName, $Password, $PeerChallenge, $PeerResponse, $AuthenticatorChallenge) = @_;

	if( 
		GenerateNTResponse($AuthenticatorChallenge, $PeerChallenge, $UserName, $Password)
			eq
		$PeerResponse
	) { return 1; } ### Should be Accept
	
	return 0;  ### Should be Rejected
}

sub DecodeMSCHAP2 {
	my ($PeerChallenge, $PeerResponse, $AuthenticatorChallenge) = split(':', shift @_);

	$PeerChallenge =~ s/(..)/0x$1, /sg;
	$PeerResponse =~ s/(..)/0x$1, /sg;
	$AuthenticatorChallenge =~ s/(..)/0x$1, /sg;

	eval " \$PeerChallenge = sprintf(\"%c\"x16,$PeerChallenge); ";
	eval " \$PeerResponse = sprintf(\"%c\"x24,$PeerResponse); ";
	eval " \$AuthenticatorChallenge = sprintf(\"%c\"x16,$AuthenticatorChallenge); ";

	return ($PeerChallenge, $PeerResponse, $AuthenticatorChallenge);
}

sub EncodeMSCHAP2 {
	my ($hash) = @_;
	my $ret;
	
	while( 1 ) {
		($_, $hash) = unpack("C a*", $hash);
		last if(!length($_));
		$ret .= sprintf("%02X", $_);
	}
	return $ret;
}


sub hex_dump_print {

        my($rest) = shift@_;
        my($char);
        my($res);

        while($rest) {
                ($char, $rest) = unpack("C a*", $rest);
                $res .= sprintf ("%02X ",$char);
        }

        return $res;
}


# Parity Correction For DES
sub DesParityCorrect {
        my ($origin) = @_;
        my ($dest);

        my ($val, $dest_bit, $sum, $dest_byte, $dest_mask, $mask) = (0, 0, 0, 0, 0x80, 0x80);
        my ($i, $j);

        for($i = 0; $i < 7; $i++) { # byte number
                $mask = 0x80;
                ($val, $origin) = unpack("C a*", $origin);
                for($j = 0; $j < 8; $j++) { # bit in byte
                        ### printf("%02x        %02x    %d      %d\n", $mask, $dest_mask, $i, $j);
                        if($val & $mask) {
                                $dest_byte += $dest_mask;
                                $sum ++;
                        }
                        $mask /= 2;
                        $dest_mask /= 2;
                        $dest_bit++;
                        if($dest_bit == 7) {
                                $dest_byte += ($sum % 2) ^ 1 ? $dest_mask :0 ;
                                $dest_mask = 0x80;
                                $dest_bit = 0;
                                ($dest) = pack("a* C", $dest, $dest_byte);
                                $dest_byte = 0;
                                $sum = 0;
                        }
                }
        }
        return $dest;
}

# Des Encryption
sub DesEncrypt {
	my ($Clear, $Key) = @_;


	$Key = DesParityCorrect($Key);

	my $cipher = new Crypt::DES $Key;
        my $Cypher = $cipher->encrypt($Clear);	
	return substr($Cypher,0,8);

}

# ChallengeResponse
sub GenerateChallengeResponse {
	my ($ChallengeHash, $PasswordHash) = @_; # 8,16

	my ($ZPasswordHash) = pack("a16 x x x x x",$PasswordHash);
	my ($Response);

	$Response .= DesEncrypt( $ChallengeHash, substr($ZPasswordHash,0,7));
	$Response .= DesEncrypt( $ChallengeHash, substr($ZPasswordHash,7,7));
	$Response .= DesEncrypt( $ChallengeHash, substr($ZPasswordHash,14,7));

	return $Response; # 24
}


# NT Response
sub GenerateNTResponse {
	my ($AuthenticatorChallenge, $PeerChallenge, $UserName, $Password) = @_;
	my ($Challenge, $PasswordHash); ## 8, 16

	$Password =~ s/(.)/$1\000/sg; ## TO UNICODE

	#print "GenerateNTResponse:AuthenticatorChallenge = ".hex_dump_print($AuthenticatorChallenge)."\n";
	#print "GenerateNTResponse:PeerChallenge = ".hex_dump_print($PeerChallenge)."\n";
	#print "GenerateNTResponse:Password = ".hex_dump_print($Password)."\n"; 

	my $ChallengeHash = GenerateChallengeHash($PeerChallenge,$AuthenticatorChallenge,$UserName);
	return GenerateChallengeResponse($ChallengeHash, sprintf("%c"x16,mdfour($Password)));
}

# ChallengeHash
sub GenerateChallengeHash {
	my ($PeerChallenge,$AuthenticatorChallenge,$UserName) = @_;
	my $sha1 = Digest::SHA1->new;
	$sha1->add($PeerChallenge);
	$sha1->add($AuthenticatorChallenge);
	$sha1->add($UserName);
	return substr($sha1->digest,0,8);
}

# AuthResponse
sub GenerateAuthenticatorResponse {
	my ($Password, $NTResponse, $PeerChallenge, $AuthenticatorChallenge, $Username) = @_;
	my ($Magic1, $Magic2, $Digest);
	my ($ChallenegeHash) = GenerateChallengeHash($PeerChallenge,$AuthenticatorChallenge,$Username); 

	$Password =~ s/(.)/$1\000/sg; ## TO UNICODE

	#print "AuthenticatorChallenge: ".hex_dump_print($AuthenticatorChallenge)."\n";
	#print "Password: ".hex_dump_print($Password)."\n";
	#print "NTResponse: ".hex_dump_print($NTResponse)."\n";

	$Magic1 = sprintf("%c"x39,0x4D, 0x61, 0x67, 0x69, 0x63, 0x20, 0x73, 0x65, 0x72, 0x76,
          0x65, 0x72, 0x20, 0x74, 0x6F, 0x20, 0x63, 0x6C, 0x69, 0x65,
          0x6E, 0x74, 0x20, 0x73, 0x69, 0x67, 0x6E, 0x69, 0x6E, 0x67,
          0x20, 0x63, 0x6F, 0x6E, 0x73, 0x74, 0x61, 0x6E, 0x74);
	$Magic2 = sprintf("%c"x41,0x50, 0x61, 0x64, 0x20, 0x74, 0x6F, 0x20, 0x6D, 0x61, 0x6B,
          0x65, 0x20, 0x69, 0x74, 0x20, 0x64, 0x6F, 0x20, 0x6D, 0x6F,
          0x72, 0x65, 0x20, 0x74, 0x68, 0x61, 0x6E, 0x20, 0x6F, 0x6E,
          0x65, 0x20, 0x69, 0x74, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6F,
          0x6E);

	my $sha1 = Digest::SHA1->new;
	$sha1->add( sprintf("%c"x16,mdfour(  sprintf("%c"x16,mdfour($Password)) )));
	$sha1->add($NTResponse);
	$sha1->add($Magic1);
	$Digest = substr($sha1->digest,0,20);

	undef $sha1;
	$sha1 = Digest::SHA1->new;
	$sha1->add($Digest);
	$sha1->add($ChallenegeHash);
	$sha1->add($Magic2);
	return "S=".EncodeMSCHAP2(substr($sha1->digest, 0 , 20));
}

1;

