#!/usr/bin/perl
use strict;

use DBI;
use Time::Local;

require "/etc/cherry/cherry.cfg";

my $logfile = "/var/log/cherry/crontab_fixed_calc.log";




my $service_date;
my $user_id;
my $service;

my $argc=@ARGV;

for(my $i=0;$i<$argc;$i++) {
        if      (@ARGV[$i] eq '--service_date' ) {
                $service_date = @ARGV[++$i];
        } elsif (@ARGV[$i] eq '--user_id' ) {
                $user_id  = @ARGV[++$i];
        } elsif (@ARGV[$i] eq '--service' ) {
                $service = @ARGV[++$i];
        } else {
                print   "Unknown options. Type '--help' for list of all options.\n";
                exit -1;
        }
}

if(length($service_date)==0 || length($user_id)==0 ||  length($service)==0) {
        print "ERROR: service_date,user_id and service required!\n";
        exit -1;
}

if( !($service eq 'telephone') && !($service eq 'channel') && !($service eq 'dry_pair') && !($service eq 'fixedcalc') ) {
	print "ERROR: service not supportted!\n";
        exit -1;
}



my $unit="month";


my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

if( !$dbh ) {
	LOG ("ERROR: Access to database failed!");
	exit;
}



my $tcurrent_str = month_end($service_date) . ' 23:59:59';
my $tcurrent = str2time($tcurrent_str);


my $srv = $service;

	my $sql="SELECT ${srv}_services.user_id FROM ${srv}_services " .
		"JOIN ${srv}_tariff_sub_plans ON ${srv}_services.oper_id=${srv}_tariff_sub_plans.oper_id AND  ".
		"${srv}_services.tariff_plan=${srv}_tariff_sub_plans.tariff_plan " .
		"WHERE ${srv}_services.user_id='$user_id' AND ${srv}_services.enabled=1  AND ".
		"(${srv}_services.date_expire IS NULL OR ${srv}_services.date_expire<'$tcurrent_str') AND ".
		"${srv}_tariff_sub_plans.unit='$unit' ";


	LOG("SQL:\n$sql\n");

	my $sth = $dbh->prepare($sql);
	if( !$sth ) {
		LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        	exit -1;
	}


	my $rv = $sth->execute;
	if( !$rv ) {
		LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        	exit -1;
	}


	if(!(open TARIFF_CLIENT, "| /usr/lib/cherry/tariffsrv/tariff_client.pl") ) {
		LOG("Can't open tariff_client");
		exit;
	}

	while ( my ($user_id) = $sth->fetchrow_array ) {
		my $cmd="fixedcalc";                   	
        	my $session_id="fixedcalc_${srv}_${tcurrent}_${user_id}_1_${unit}"; 
        	my $service_type=$srv;     
        	my $session_begin=$tcurrent;		        
        	my $session_long=1;	         	
        	my $host="billing";	               

		my $req = "$cmd\t$session_id\t$service_type\t$session_begin\t$session_long\t$user_id\t\t$host\n"; 
		LOG ("REQ: $req");

		if(!(print TARIFF_CLIENT $req)) {
			LOG("Can't send request to tariff_client.");
			exit;
		}

	}



close (TARIFF_CLIENT);
$dbh->disconnect;
exit;

##################################################################################

sub LOG {
        my ($msg) = @_;
        my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                localtime(time);

        my($date) = sprintf "%02d.%02d.%04d %02d:%02d:%02d", $mday, $mon+1, $year+1900, $hour, $min, $sec;

        $msg = $date."\t".$msg;

        open(OUT,">>$logfile");
        print OUT $msg."\n";
        print $msg."\n";
        close(OUT);
}

#################################################################################
sub time2str {
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(shift @_);
        $mon++;
        $year += 1900;
        return sprintf("%.4i-%.2i-%.2i %.2i:%.2i:%.2i",$year,$mon,$mday,$hour,$min,$sec);
}

######################################################################################3
sub str2time {
        my($date) = @_;
        my ($year, $mon, $mday, $hour, $min, $sec) = $date =~ /^(\d\d\d\d)-(\d\d)-(\d\d)(?: (\d\d)\:(\d\d)(?:\:(\d\d))?)?$/;
        return timelocal($sec, $min, $hour, $mday, $mon - 1, $year - 1900);
}
######################################################################################
sub month_begin {
        my ($date) =@_; #In 'YYYY-MM-DD' format
        if($date =~ /(\d\d\d\d)\-(\d\d)\-(\d\d)/ ){
                        return $1 . "-" . $2 . "-01";
        }

        return ;
}

sub month_end {
        my ($date) =@_; #In 'YYYY-MM-DD' format
        my $month_begin = month_begin($date);
        if($month_begin) {
		my $month_end;
                chop($month_end=`date -j -v+1m -v-1d -f "%Y-%m-%d" $month_begin "+%Y-%m-%d"`);
                return $month_end;
        }
        return ;
}

sub previous_month_begin {
        my ($date) =@_; #In 'YYYY-MM-DD' format
        my $month_begin = month_begin($date);
        if($month_begin) {
		my $previous_month_begin;
                chop($previous_month_begin=`date -j -v-1m -f "%Y-%m-%d" $month_begin "+%Y-%m-%d"`);
                return $previous_month_begin;
        }
        return ;
}
