package pay_card;

use strict;
no strict "refs";

use DBI;
use Data::Dumper;

my %services;
my %phone_services;



sub reload; 
sub do_cmd; 
sub get_service;
sub get_phone_service;

###########################################################################################
sub get_service {
	my ($oper_id,$user_id,$session_begin) =@_;

	foreach my $srv ( sort{$b->{'date_create'} cmp $a->{'date_create'} } 
				@{ $services{$oper_id}{$user_id} }) 
	{

        	if ($session_begin ge $srv->{'date_create'}) {
			if (! defined($srv->{'date_expire'}) ) {
				return $srv;
			} else {
				if( $session_begin le $srv->{'date_expire'} ) {
					return $srv;
				}
                	}
		}

        }

	return;
}
###########################################################################################
sub get_phone_service {
        my ($oper_id,$user_id,$session_begin) =@_;

        foreach my $srv ( sort{$b->{'date_create'} cmp $a->{'date_create'} }
                                @{ $phone_services{$oper_id}{$user_id} })
        {

                if ($session_begin ge $srv->{'date_create'}) {
                        if (! defined($srv->{'date_expire'}) ) {
                                return $srv;
                        } else {
                                if( $session_begin le $srv->{'date_expire'} ) {
                                        return $srv;
                                }
                        }
                }

        }

        return;
}
###################################################################################
sub reload {
	my ($dbh,$service) = @_;

	my $sth;
	my $rv;
	my $sql;


        ##### load pay card services ###########################

        $sql = 	"SELECT  oper_id, ".
		" client_id, ".
		" user_id, ".
		" enabled, ".
		" date_create, ".
		" date_active, ".
		" date_expire, ".
		" password ".
		" FROM pay_card_services ";

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);
        $sth = $dbh->prepare($sql);
        if( !$sth ) {
                main::LOG ("ERROR: SQL request syntax error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
                main::LOG ("ERROR: SQL request execution error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        while( my ( $oper_id, $client_id, $user_id, $enabled,$date_create, 
		$date_active,$date_expire,$password ) = $sth->fetchrow_array)
	{
		push @{ $services{$oper_id}{$user_id} },
			{
				oper_id => $oper_id,
				client_id => $client_id,
				user_id => $user_id, 
				enabled => $enabled,
				date_create => $date_create,
				date_active => $date_active,
				date_expire => $date_expire,
				password => $password,
			};
	};


	### load telephone services ####


        $sql =  "SELECT  oper_id, ".
                " client_id, ".
                " user_id, ".
                " enabled, ".
                " date_create, ".
                " date_active, ".
                " date_expire ".
                " FROM telephone_services ";

        main::LOG("SQL:\n$sql\n") if($main::DEBUG>1);
        $sth = $dbh->prepare($sql);
        if( !$sth ) {
                main::LOG ("ERROR: SQL request syntax error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        $rv = $sth->execute;
        if( !$rv ) {
                main::LOG ("ERROR: SQL request execution error: $! $@") if ($main::DEBUG>0);
                return ;
        }

        while( my ( $oper_id, $client_id, $user_id, $enabled,$date_create,
                $date_active,$date_expire ) = $sth->fetchrow_array)
        {
                push @{ $phone_services{$oper_id}{$user_id} },
                        {
                                oper_id => $oper_id,
                                client_id => $client_id,
                                user_id => $user_id,
                                enabled => $enabled,
                                date_create => $date_create,
                                date_active => $date_active,
                                date_expire => $date_expire,
                        };
        };


	return 'OK';

}

##################################################################################
sub do_cmd { 


	my (
		$dbh,
                $cmd, # auth,update
                $query_id,
                $service_type,
                $session_begin,
                $session_long,
                $user_id,
                $password,
                $nas_id,
                $src_major,
                $src_minor,
                $dst_major, #     
                $dst_minor,
                $bytes_in,
                $bytes_out,
                $via,
                $disconnect_cause,
                $voice_quality,


	) = @_;


        my ($oper_id,$nas_type) = &common::get_oper_id($nas_id);
        if(!defined($oper_id)) {
                main::LOG ("ERROR: oper_id not found (nas_id=$nas_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\toper_id not found (nas_id=$nas_id)\n";
        }



        my $srv=get_service($oper_id,$user_id,$session_begin);
        if(!defined($srv)) {
        	main::LOG ( "ERROR: user_id not found " .
                	"(oper_id=$oper_id, user_id=$user_id, session_begin=$session_begin)") 
			if ($main::DEBUG>0);

                return "ERROR\t$query_id\tuser_id not found" .
                	"(oper_id=$oper_id, user_id=$user_id, session_begin=$session_begin)"."\n";

        }


        my $card_client_id = $srv->{'client_id'};
        if(!defined($card_client_id)) {
                main::LOG ("ERROR: client_id not found (oper_id=$oper_id, user_id=$user_id)"
                        ) if ($main::DEBUG>0);

                return "ERROR\t$query_id\tclient_id not found (oper_id=$oper_id, " .
                        "user_id=$user_id)\n";
        }




        my ($card_balance,$card_balance_currency_type) =
                        &common::get_client_balance($dbh,$oper_id,$card_client_id);

        if(!defined($card_balance) ) {
                main::LOG ("ERROR: can't get balance (oper_id=$oper_id, " .
                "client_id=$card_client_id)") if ($main::DEBUG>0);

                return "ERROR\t$query_id\tcan't get balance (oper_id=$oper_id, " .
                        "client_id=$card_client_id)\n";
        }

        if(!defined($card_balance_currency_type) ) {
                main::LOG ("ERROR: balance's currency_type not found ".
                "(oper_id=$oper_id, client_id=$card_client_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\tbalance's currency_type not found ".
                        "(oper_id=$oper_id, client_id=$card_client_id)\n";
        }

	##### auth 1

                my $enabled = $srv->{'enabled'};
                if(!$enabled) {
                        main::LOG ("ERROR: user blocked (oper_id=$oper_id, " .
                        " user_id=$user_id)") if ($main::DEBUG>0);

                        return "ERROR\t$query_id\tuser blocked (oper_id=$oper_id, " .
                                "user_id=$user_id)\n";
                }

		if($srv->{ani}) {
			my $ani = $srv->{ani};
			if($src_major !~ /${ani}$/ ) {
				main::LOG ("ERROR: ani authorization failed " 	.
					"(src_addr_major=$src_major, ani=$ani )" ) 
					if ($main::DEBUG>0);

				return "ERROR\t$query_id\tani authorization failed "  .
                                        "(src_addr_major=$src_major, ani=$ani )\n";

			}	

		}

		my $srv_password = $srv->{'password'};

			if(!($password eq $srv_password )) {
				main::LOG ("ERROR: authorization failed " .
				"(oper_id=$oper_id, user_id=$user_id)") if ($main::DEBUG>0);

				return "ERROR\t$query_id\tauthorization failed ".
				"(oper_id=$oper_id, user_id=$user_id)\n";
			}


                if(($card_balance )<= 0  ) {
                        main::LOG ("ERROR: negative balance") if ($main::DEBUG>0);
                        return "ERROR\t$query_id\tnegative balance\n";
                }


		#     
        	if (!$dst_major) {
        		my $max_time = -1;
        		my ($gender_h,$gender_l) = &common::get_currency_gender ($card_balance_currency_type);

        		if(!defined($gender_h) || !defined($gender_l)) {
                        	main::LOG ("ERROR: Can't get currency gender " .
                        	"(balance_currency_type=$card_balance_currency_type)") if ($main::DEBUG>0);

                        	return "ERROR\t$query_id\tcan't get currency gender " .
                        	"(balance_currency_type=$card_balance_currency_type)\n";
        		}

        		return "OK\t$query_id\t$card_balance\t$max_time\t$card_balance_currency_type\t$gender_h\t$gender_l\n";
		}




	#### auth 2 ###########

	my $phone=$dst_major;

        my $phone_srv=get_phone_service($oper_id,$phone,$session_begin);
        if(!defined($phone_srv)) {
                main::LOG ( "ERROR: user_id not found " .
                        "(oper_id=$oper_id, user_id=$phone, session_begin=$session_begin)")
                        if ($main::DEBUG>0);

                return "ERROR\t$query_id\tuser_id not found" .
                        "(oper_id=$oper_id, user_id=$phone , session_begin=$session_begin)"."\n";

        }


        my $phone_client_id = $phone_srv->{'client_id'};
        if(!defined($phone_client_id)) {
                main::LOG ("ERROR: client_id not found (oper_id=$oper_id, user_id=$phone)"
                        ) if ($main::DEBUG>0);

                return "ERROR\t$query_id\tclient_id not found (oper_id=$oper_id, " .
                        "user_id=$phone)\n";
        }




        my ($phone_balance,$phone_balance_currency_type) =
                        &common::get_client_balance($dbh,$oper_id,$phone_client_id);

        if(!defined($phone_balance) ) {
                main::LOG ("ERROR: can't get balance (oper_id=$oper_id, " .
                "client_id=$phone_client_id)") if ($main::DEBUG>0);

                return "ERROR\t$query_id\tcan't get balance (oper_id=$oper_id, " .
                        "client_id=$phone_client_id)\n";
        }

        if(!defined($phone_balance_currency_type) ) {
                main::LOG ("ERROR: balance's currency_type not found ".
                "(oper_id=$oper_id, client_id=$phone_client_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\tbalance's currency_type not found ".
                        "(oper_id=$oper_id, client_id=$phone_client_id)\n";
        }


        if ($cmd eq 'auth') {
                        my $max_time = -1;
                        my ($gender_h,$gender_l) = &common::get_currency_gender ($phone_balance_currency_type);

                        if(!defined($gender_h) || !defined($gender_l)) {
                                main::LOG ("ERROR: Can't get currency gender " .
                                "(balance_currency_type=$phone_balance_currency_type)") if ($main::DEBUG>0);

                                return "ERROR\t$query_id\tcan't get currency gender " .
                                "(balance_currency_type=$phone_balance_currency_type)\n";
                        }

                        return "OK\t$query_id\t$phone_balance\t$max_time\t$phone_balance_currency_type\t$gender_h\t$gender_l\n";
        }



	####### update  ##############

	if( ! $dbh->do("UPDATE base_clients SET balance = 0 WHERE client_id='$card_client_id' AND oper_id='$oper_id'"  )) {
		main::LOG ("ERROR: can't update balance (oper_id=$oper_id, client_id=$card_client_id)") if ($main::DEBUG>0);
		return "ERROR\t$query_id\tcan't update balance (oper_id=$oper_id, client_id=$card_client_id)\n";
	}

	if( ! $dbh->do("UPDATE pay_card_services SET date_active = 'now', enabled = 0, descr = ' : $phone_client_id ($phone)' WHERE user_id = '$user_id' AND oper_id='$oper_id'"  )) {
		main::LOG ("ERROR: can't update pay_card_services (oper_id='$oper_id',  user_id = '$user_id') ") if ($main::DEBUG>0);
		return "ERROR\t$query_id\tcan't update pay_card_services (oper_id='$oper_id',  user_id = '$user_id') \n";
	}


	if( ! $dbh->do("INSERT INTO payment_payments (client_id,oper_id,payment_sum,payment_sum_added,payment_type,currency_type,descr,payment_date,income_date) VALUES ('$card_client_id','$oper_id', ". -1 * $card_balance .",". -1 * $card_balance .",'transfer','$card_balance_currency_type',' : $phone_client_id ($phone)', 'now', 'now')"  )) {
		main::LOG ("ERROR: can't insert payment (oper_id=$oper_id, client_id=$card_client_id) ") if ($main::DEBUG>0);
		return "ERROR\t$query_id\tcan't insert payment (oper_id=$oper_id, client_id=$card_client_id) \n";
	}


	my $payment_sum = common::translate_currency ($oper_id,$card_balance_currency_type,$phone_balance_currency_type,$session_begin,$card_balance) ;
	if(!$payment_sum) {
		
                        main::LOG ("ERROR: Can't translate currency (oper_id=$oper_id, " .
                        "card_balance_currency_type=$card_balance_currency_type, " .
                        "phone_balance_currency_type=$phone_balance_currency_type, session_begin=$session_begin," .
                        " card_balance=$card_balance)") if ($main::DEBUG>0);

                        return "ERROR\t$query_id\tcan't translate currency (oper_id=$oper_id, " .
                        "card_balance_currency_type=$card_balance_currency_type, " .
                        "phone_balance_currency_type=$phone_balance_currency_type, session_begin=$session_begin," .
                        " card_balance=$card_balance)\n";

	}

	if( ! $dbh->do("INSERT INTO payment_payments (client_id,oper_id,payment_sum,payment_sum_added,payment_type,currency_type,descr,payment_date,income_date) VALUES ('$phone_client_id','$oper_id', $payment_sum, $payment_sum,'transfer','$phone_balance_currency_type',' : $card_client_id ($user_id)', 'now', 'now')"  )) {
                main::LOG ("ERROR: can't insert payment (oper_id=$oper_id, client_id=$card_client_id) ") if ($main::DEBUG>0);
                return "ERROR\t$query_id\tcan't insert payment (oper_id=$oper_id, client_id=$card_client_id) \n";

	}

	if( ! $dbh->do("UPDATE base_clients SET balance = balance + ($payment_sum) WHERE client_id='$phone_client_id' AND oper_id='$oper_id'"  )) {
                main::LOG ("ERROR: can't update balance (oper_id=$oper_id, client_id=$card_client_id)") if ($main::DEBUG>0);
                return "ERROR\t$query_id\tcan't update balance (oper_id=$oper_id, client_id=$card_client_id)\n";

	}

	$srv->{enabled}=0;
	$srv->{date_active}=1;


	
	$dbh->commit;


        #update balance is ok. get it again.

       	($phone_balance) = &common::get_client_balance($dbh,$oper_id,$phone_client_id);
       	if(!defined($phone_balance) ) {
       		main::LOG ("ERROR: can't get balance (oper_id=$oper_id, ".
			"client_id=$phone_client_id)") if ($main::DEBUG>0);

                        return "ERROR\t$query_id\tcan't get balance (oper_id=$oper_id, " .
			"client_id=$phone_client_id)\n";
        }

	my $max_time = -1;

        my ($gender_h,$gender_l) = &common::get_currency_gender ($phone_balance_currency_type);

        if(!defined($gender_h) || !defined($gender_l)) {
                        main::LOG ("ERROR: Can't get currency gender " .
                        "(balance_currency_type=$phone_balance_currency_type)") if ($main::DEBUG>0);

                        return "ERROR\t$query_id\tcan't get currency gender " .
                        "(balance_currency_type=$phone_balance_currency_type)\n";
        }



        return "OK\t$query_id\t$phone_balance\t$max_time\t$phone_balance_currency_type\t$gender_h\t$gender_l\n";

}
#########################################################################
1;

