#!/usr/bin/perl
use strict;

use DBI;

require "/etc/cherry/cherry.cfg";

my $logfile = "/var/log/cherry/recalc.log";


my $from_date;
my $till_date;

my $oper_id;
my $client_id;
my $user_id;
my $service;

my $argc=@ARGV;

for(my $i=0;$i<$argc;$i++) {
	if	(@ARGV[$i] eq '--from_date' ) {
		$from_date = @ARGV[++$i];
	} elsif	(@ARGV[$i] eq '--till_date' ) {
                $till_date = @ARGV[++$i];
        } elsif (@ARGV[$i] eq '--user_id' ) {
                $user_id  = @ARGV[++$i];
        } elsif (@ARGV[$i] eq '--service' ) {
                $service = @ARGV[++$i];
	} elsif (@ARGV[$i] eq '--help' ) {
		print 	"This is Cherry Recalc program.\n\n" .
			"Usage:\n" .	
			"recalc --from_date <timestamp> --till_date <timestamp> [options]\n\n" .
			"Options:\n".
			"--from_date <timestamp>\t\tsession_begin >= this date\n" .
			"--till_date <timestamp>\t\tsession_begin <= this date\n" .
                        "--user_id <string>\t\tuser's identifier (oper_id and client_id required)\n" .
                        "--service <string>\t\tservice type's identifier\n" .
			"--help\t\t\t\tthis help\n"; 
	} else {
		print	"Unknown options. Type '--help' for list of all options.\n";
		exit -1;
	}
}

if(length($from_date)==0 || length($till_date)==0 || length($user_id)==0) {
	print "ERROR: from_date,till_date and user_id required!\n";
	exit -1;
}



my $where;

if(length($from_date)>0) {
	$where.=" AND session_begin >= '$from_date'";
}
if(length($till_date)>0) {
	$where.=" AND session_begin <= '$till_date'";
}
if (length($user_id)>0) {
	$where.=" AND user_id = '$user_id'";
}
if (length($service)>0) {
	$where.=" AND session_type = '$service'";
}

$where =~ s/^\ AND//;

my $sql = " SELECT session_id FROM base_sessions WHERE $where ORDER BY session_begin,session_id";


my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

if( !$dbh ) {
	LOG ("ERROR: Access to database failed!");
	exit;
}



LOG("SQL:\n$sql\n");


my $sth = $dbh->prepare($sql);
if( !$sth ) {
	LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        exit -1;
}


my $rv = $sth->execute;
if( !$rv ) {
	LOG (sprintf("SQLERROR: %s", $dbh->errstr));
        exit -1;
}


if(!(open TARIFF_CLIENT, "| /usr/lib/cherry/tariffsrv/tariff_client.pl") ) {
	LOG("Can't open tariff_client");
	exit;
}

my $count=0;
my @sessions=();

while ( my ($session_id) = $sth->fetchrow_array ) {
	$count++;
	push @sessions, $session_id;
}
	
$|=1;

print ("\nTOTAL_REQUEST $count\n");

$count=0;

foreach my $session_id (@sessions) {
	$count++;
	my $req = "recalc\t$session_id\tDESCR=(REQUEST_NUMBER $count)\n"; 
	if(!(print TARIFF_CLIENT $req)) {
		LOG("Can't send request to tariff_client.");
		exit;
	}
}



close (TARIFF_CLIENT);
$dbh->disconnect;
exit;

##################################################################################

sub LOG {
        my ($msg) = @_;
        my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                localtime(time);

        my($date) = sprintf "%02d.%02d.%04d %02d:%02d:%02d", $mday, $mon+1, $year+1900, $hour, $min, $sec;

        $msg = $date."\t".$msg;

        open(OUT,">>$logfile");
        print OUT $msg."\n";
        print $msg."\n";
        close(OUT);
}

