#!/usr/bin/perl 
package main;

use strict;
no strict "refs";

use Socket;
use FileHandle;
use DBI;
use Time::HiRes;
use Data::Dumper;

require "/etc/cherry/cherry.cfg";
require "/usr/lib/cherry/tariffsrv/common.pl";
require "/usr/lib/cherry/tariffsrv/recalc.pl";


$main::DEBUG = 2;		#debug level; 0 - off; 1,2 - on

$main::NOREBOOT =0;		# don't reboot tarifficator if history changes

my $DEFAULT_CHECK_HISTORY_TIMEOUT = 60 ; 
my $DEFAULT_PORT = 2347;

my $check_history_timeout;	# 
my $history_last_update; 	# timestamp of last update of the history table (for reloading server)


my $ticks_history=0;
my $log_flush_time = 60;

my $ticks_log =0;

my $log_file="tariffsrv.log";

$main::LOG_DATA="";


my $history_dbh;

$main::netflow_dbh;

### Used tables ######################
# base_history
######################################


sub LOG ;
sub get_db_connection ;
sub dummy_handler ;
sub chld_handler ;
sub hup_handler ;
sub alarm_handler ;
sub get_history_last_update ;
sub access_check ;
sub get_cfg_param ;
sub do_cmd ;

#### File handlers 
# LOG_FILE;
# CLIENT;
# SERVER;

##########################################################################
sub LOG {
        my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	my($tt) = Time::HiRes::time;
        my($date) = sprintf "%.3f %02d.%02d.%04d %02d:%02d:%02d", $tt, $mday, $mon+1, $year+1900, $hour, $min, $sec;

        foreach my $logstr (@_) {
                $main::LOG_DATA .= "$$\t$date\t$logstr\n";
        }

}
##########################################################################
sub LOG_FLUSH {
        if($main::LOG_DATA) {
                open(LOG_FILE, ">> $log_file") || return;
                print LOG_FILE $main::LOG_DATA;
                print $main::LOG_DATA;
                close(LOG_FILE);
                $main::LOG_DATA = "";
        }
}
##########################################################################
sub get_db_connection {
        my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

        if( !$dbh ) {
                LOG ("get_db_connection\tERROR: Access to database failed!") if ($main::DEBUG>0);
        } else {
        	my $rv1=$dbh->do("SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;");
                if( !$rv1 ) {
                        LOG (sprintf("SET TR ISO SQLERROR: %s", $dbh->errstr)) if ($main::DEBUG>0);
                }
	}


        return $dbh;

}
##########################################################################
sub get_netflow_db_connection {
        my $dbh = DBI->connect("dbi:$CFG::DB_TYPE:dbname=$CFG::DB_NAME_NETFLOW;host=$CFG::DB_HOST",
        "$CFG::DB_USER", "$CFG::DB_USER_PASSWORD", { RaiseError => 0, AutoCommit => 0, PrintError => 0  });

        if( !$dbh ) {
                LOG ("get_netflow_db_connection\tERROR: Access to database failed!") if ($main::DEBUG>0);
        }

        return $dbh;

}

##########################################################################
sub dummy_handler { LOG_FLUSH; }
##########################################################################
sub child_handler {
	#wait end of the child process 
        my $waitedpid = wait;
        $SIG{CHLD} = \&child_handler;
        LOG ("riaped $waitedpid" . ($? ? " with exit $?" : '')) if ($main::DEBUG>0);
	LOG_FLUSH;
}
##########################################################################
sub hup_handler {
	close CLIENT;
	LOG("SIGHUP received, processign stopped!");
	LOG_FLUSH;
	exit(1);
}
##########################################################################
sub alarm_handler_parent {

        $ticks_log++;
        if($ticks_log >= $log_flush_time) {
                LOG_FLUSH;
                $ticks_log=0;
        }

        alarm(1);

}
########################################################################
sub alarm_handler_child {

	my $ts=Time::HiRes::time;

        #$ticks_log++;
        #if($ticks_log*2 >= $log_flush_time) {
        #        LOG_FLUSH;
        #        $ticks_log=0;
        #}

        $ticks_history++;
        if($ticks_history*2 >= $check_history_timeout ) {
                $ticks_history=0;

                my($history) = get_history_last_update;
                if(defined($history) && !($history_last_update eq $history) && ! $main::NOREBOOT) {
			main::LOG ("Reboot now (history changed) $history,$history_last_update,$main::NOREBOOT");

                        $history_last_update = $history;

        		close CLIENT;
        		LOG_FLUSH;
        		exit(2);
                }

        }

	#LOG_FLUSH;
	#LOG ("Timer=". sprintf("%0.4f", Time::HiRes::time - $ts) );


	alarm(5);
}
########################################################################
sub get_history_last_update {
        #my $dbh = get_db_connection || return;
	#if($history_dbh) {
        	my ($history) = $history_dbh->selectrow_array("SELECT max(date) FROM base_history WHERE reboot=1");
        	#$dbh->disconnect;
        	return $history;
	#}
}
########################################################################
sub access_check {
	#check host authorised
        my ($ip,$hosts) = @_;
        $ip =~ s/\./\\\./g;
        if (!($hosts =~ /$ip/g) && !($hosts eq "any") )  {
                return 0;
        }
        return 1;
}
#########################################################################
sub get_cfg_param {
        my ($port,$hosts,$interface,$max_conn,$check_history_timeout,$services,$log_file);

        if(defined($CFG::TARIFFICATOR->{default_port})) {
                $port = $CFG::TARIFFICATOR->{default_port};
        } else {
                $port = $DEFAULT_PORT;
        }

        if (defined($CFG::TARIFFICATOR->{hosts})) {
                $hosts = $CFG::TARIFFICATOR->{hosts};
        } else {
                $hosts = "any";
        }

        if (defined($CFG::TARIFFICATOR->{interface})) {
                $interface = $CFG::TARIFFICATOR->{interface};
        } else {
                $interface = "0.0.0.0";
        }

        if (defined($CFG::TARIFFICATOR->{max_conn})) {
                $max_conn = $CFG::TARIFFICATOR->{max_conn};
        } else {
                $max_conn = SOMAXCONN;
        }

	if (defined($CFG::TARIFFICATOR->{check_history_timeout})) {
		$check_history_timeout = $CFG::TARIFFICATOR->{check_history_timeout};		
	} else {
		$check_history_timeout = $DEFAULT_CHECK_HISTORY_TIMEOUT;
	}


	$services = $CFG::TARIFFICATOR->{services};
	$log_file = $CFG::TARIFFICATOR->{log_file};

        LOG (sprintf "Config param: port = %s, hosts = %s,interface = %s, max_conn = %s, check_history_timeout = %s, services=%s, log_file=%s\n", $port,$hosts,$interface,$max_conn,$check_history_timeout,$services,$log_file) if ($main::DEBUG>0);

        return ($port,$hosts,$interface,$max_conn,$check_history_timeout,$services,$log_file);
}

sub term_handler {
	LOG("SIGTERM received!"); 
	LOG_FLUSH; 
	close SERVER; 
	exit(3);

}

sub pipe_handler {
    LOG ("SIG PIPE received!");
    LOG_FLUSH;
    exit(4);
}

##################################################################################
##################################################################################
##################################################################################


my ($port,$hosts,$interface,$max_conn,$history_timeout,$used_services,$log_file_);

($port,$hosts,$interface,$max_conn,$history_timeout,$used_services,$log_file_)= get_cfg_param;
$log_file = $log_file_;

socket(SERVER, PF_INET, SOCK_STREAM, getprotobyname('tcp') )  || LOG_FLUSH && die "socket: $!";
setsockopt(SERVER, SOL_SOCKET, SO_REUSEADDR, 1)   || LOG_FLUSH && die "setsockopt: $!";
bind(SERVER, sockaddr_in($port, inet_aton($interface))) ||  LOG_FLUSH && die "bind: $!";
listen(SERVER,$max_conn) || LOG_FLUSH && die "listen: $!";

$SIG{CHLD} = \&child_handler; 
$SIG{HUP}  = \&dummy_handler; 


$check_history_timeout = $history_timeout;

$SIG{ALRM} = \&alarm_handler_parent;
alarm_handler_parent(); 

#$SIG{TERM} =  \&term_handler;
#$SIG{INT} =   \&term_handler; 




LOG ("tariffic server started on port $port") if($main::DEBUG>0);
LOG_FLUSH;

while (1) {
	sleep 1;
	my $paddr = accept(CLIENT,SERVER) || LOG_FLUSH && die "accept: $!";
	if (!$paddr) {
		#LOG("paddr not defined!");
		#LOG_FLUSH();
		sleep 1;
		next;
	};
        my ($port,$iaddr) = sockaddr_in($paddr);
        my $name = gethostbyaddr($iaddr,AF_INET);
	my $ip = inet_ntoa($iaddr);

	if(!access_check($ip,$hosts)) {
		#host not authorised
		LOG (sprintf "access denied - connection from %s [%s] at port %d",$name,$ip,$port) if ($main::DEBUG>0);
		LOG_FLUSH;
		close CLIENT;
		next;
	}

        LOG (sprintf "connection from %s [%s] at port %d", $name,$ip,$port) if($main::DEBUG>0);
	LOG_FLUSH;

        my $pid;
        if (!defined($pid = fork)) {
                LOG ("cannot fork: $!") if ($main::DEBUG>0);
		LOG_FLUSH;
		sleep 1;
        } elsif ($pid) { # I'm the parent
		close CLIENT;
                LOG ("began $pid") if ($main::DEBUG>0) ;
		LOG_FLUSH;
		sleep 1;
        } else { #  else I'm the child

		close SERVER;

		$SIG{HUP} = \&hup_handler;
		$SIG{PIPE}  = \&pipe_handler;




		my $dbh ;
		

		if(!($dbh= get_db_connection)) {
			LOG("Cannot get DB connection");
			LOG_FLUSH;
			exit (-1);
		}

		if(!($history_dbh= get_db_connection)) {
			LOG("Cannot get DB connection for history");
			LOG_FLUSH;
			exit(-1);
		}

		if(!($main::netflow_dbh = get_netflow_db_connection)) {
			LOG("Cannot get DB connection for netflow");
			LOG_FLUSH();
			exit(-1);
		}

		$history_last_update = &get_history_last_update;


		&common::reload($dbh) || LOG_FLUSH && die "$@ $!";

		
        	foreach my $srv (split(' ', $used_services)) {
                	require "/usr/lib/cherry/tariffsrv/${srv}.pl";
                	&{$srv . '::reload'} ($dbh,$srv) || 
				LOG_FLUSH && die "$@ $! ". 
				"/usr/lib/cherry/tariffsrv/${srv}.pl";

        	}


		$SIG{ALRM} = \&alarm_handler_child;
		alarm_handler_child();

        	CLIENT->autoflush(1);
        	print CLIENT "Cherry (R) tarifficator server\n";
		print CLIENT ">\n";

		my $cmd;
        	while ($cmd=<CLIENT>){
			my $begin_query = Time::HiRes::time;

			$cmd =~ s/(\n|\r)//g;
			LOG ("REQ:$cmd") if($main::DEBUG>0);
			LOG_FLUSH;


			my $answer="";

			if($cmd eq "?") {
        			print CLIENT "auth\nupdate\nfinish\nrecalc\nnetflow\nfixedcalc\nexit\n";
			} elsif($cmd eq "reboot_enable") {
				$main::NOREBOOT = 0;
				print CLIENT "REBOOT IS ENABLE\n";
			} elsif($cmd eq "reboot_disable") {
				$main::NOREBOOT = 1;
				print CLIENT "REBOOT IS DISABLE\n";
			} elsif($cmd =~ /(^auth)|(^update)|(^finish)|(^recalc)|(^netflow)|(^fixedcalc)/) {
        			my (    $cmd_,
                			$query_id,
                			$service_type,
                			$session_begin,
                			$session_long,
                			$user_id,
                			$password,
                			$nas_id,
                			$src_major,
                			$src_minor,
                			$dst_major,
                			$dst_minor,
                			$bytes_in,
                			$bytes_out,
                			$via,
                			$disconnect_cause,
                			$voice_quality,
					$direction_dynamic_ip,
        			) = split ("\t", $cmd );


				
				$session_begin = common::time2str($session_begin);


                                if($service_type eq 'pay_card_payment' ) {
                                        $cmd_ = 'update';
                                        $service_type = 'pay_card';
                                }


                                if($service_type eq 'telephone_info' ) {
                                        $cmd_ = 'info';
                                        $service_type = 'telephone';
                                }

				

                                if($cmd_ eq 'recalc') {
                                        my $ret;

                                       ($ret,
                                        $query_id,
                                        $service_type,
                                        $session_begin,
                                        $session_long,
                                        $user_id,
                                        $nas_id,
                                        $src_major,
                                        $src_minor,
                                        $dst_major,
                                        $dst_minor,
                                        $bytes_in,
                                        $bytes_out,
                                        $via,
                                        $disconnect_cause,
                                        $voice_quality,
                                       ) = tariff_recalc::get_tariff_data($dbh,$query_id);

                                        tariff_recalc::prepare_recalc($dbh,$query_id);

                                        $password = '';
					if($query_id =~ /^fixedcalc/) {
						$cmd_="fixedcalc";
					} else {
                                        	$cmd_="update";
					}
                                }


                                if($service_type eq 'opers') {
                                        my $answer="";

                                        if($common::gnas_list{$nas_id}) {
                                                if($common::gnas_list{$nas_id}->{'password'} eq $password) {
                                                        $answer="OK\t$query_id\n";
                                                } else {
                                                        $answer="ERROR\t$query_id\tauth_oper authorization failed!\n";
                                                }
                                        } else {
                                                $answer="ERROR\t$query_id\toper_id,nas_id($user_id,$nas_id) not found!\n";
                                        }
                                        LOG ("ANSWER: $answer") if($main::DEBUG>0);
                                        LOG_FLUSH;
                                        print CLIENT $answer;
                                        print CLIENT ">\n";
                                        next;
                                }




				if ($service_type && $used_services=~ /$service_type/)  {

					my $tarifficator_module = $service_type;
					if($cmd_ eq 'netflow') {
						$tarifficator_module = 'netflow';
					}

        				$answer=( &{$tarifficator_module.'::do_cmd'} (
                        		$dbh,
                        		$cmd_,
                        		$query_id,
                        		$service_type,
                        		$session_begin,
                        		$session_long,
                        		$user_id,
                        		$password,
                        		$nas_id,
                        		$src_major,
                        		$src_minor,
                        		$dst_major,
                        		$dst_minor,
                        		$bytes_in,
                        		$bytes_out,
                        		$via,
                        		$disconnect_cause,
                        		$voice_quality,
					$direction_dynamic_ip,
                			)
        				);

					#if($cmd_ eq 'netflow' && !$user_id ) {
						#
						# tsss ...
					#} else {
						LOG ("ANSWER: $answer") if($main::DEBUG>0);
						LOG_FLUSH;
						my $rc = syswrite(CLIENT, $answer, length($answer));
						if($rc < 0) {
							LOG("WRITE ERROR: $rc $!");
						}
						
					#}


				} else {
                                        LOG ("ERROR: unknown service_type ($service_type)") if ($main::DEBUG>0);
					LOG_FLUSH;
					my $mb = "ERROR\t$query_id\tunknown service_type ($service_type)\n";
                                        syswrite(CLIENT, $mb, length($mb));
				}

			} elsif($cmd eq "exit"){
				syswrite(CLIENT,"OK\n", 3);
				close CLIENT;
				LOG_FLUSH;
        			exit(4);
			} else {
				print CLIENT "ERROR\tUnknown command. Type \"?\" for help.\n"
			}
			syswrite(CLIENT, ">\n", 2);

			LOG (sprintf("PROCESSING_TIME=%f\t$cmd\t__ANSWER__\t$answer", Time::HiRes::time - $begin_query) ) if ($main::DEBUG>0);
			LOG_FLUSH;
		}
		LOG ("CLIENT EOF");
		LOG_FLUSH;
		exit(5);
	}

}

##################################################################################
1;
