#
# This tcl script does AAA AUTHENTICATION and AAA ACCOUNTING for remote voip 
# callis initiated by voip peer gateways or IP phones. It is based on Cisco's 
# original remote_ip.tcl code.
#
# Rewritten by Ruslan Zalata, November 2002.
#
# CAUTION: You ough to change AAA AUTHENTICATION password to something real
#          before running this. See passwd variable in acct_Setup procedure. 

proc init { } {
    global param

    set param(interruptPrompt) true
    set param(abortKey) *
    set param(terminationKey) #

}

proc act_Setup { } {
    global dest
    global ani
    global beep
    global ipaddr
    global passwd

    set passwd "VivaVoIP"

    set beep 0


    set ipaddr [infotag get leg_remoteipaddress leg_incoming]
    if {$ipaddr == ""} {
	puts "Call [infotag get leg_incoming] has NULL remote IP address\n"
	call close
	return
    }

    set dest ""
    set ani ""

    if { [infotag get leg_isdid] } {
	set dest [infotag get leg_dnis]
	set ani [infotag get leg_ani]
    }

#    aaa authenticate $ipaddr $passwd av-send
    set av-send(h323-ivr-out,10) "remote_ip:true"
    aaa authorize $ipaddr $passwd $ani $dest leg_incoming av-send
}

proc act_Authenticated { } {
    global ipaddr

    set status [infotag get evt_status]
    if {$status != "ao_000"} {
	puts "Call [infotag get leg_incoming] authen failed,ip=$ipaddr,code=$status\n"
	call close
	return
    }

    if { [infotag get leg_isdid] } {
	set dest [infotag get leg_dnis]
	leg proceeding leg_incoming
	leg setup $dest callInfo leg_incoming
	fsm setstate PLACECALL
    } else {
        leg setupack leg_incoming
	playtone leg_incoming tn_dial  

	set param(dialPlan) true
	leg collectdigits leg_incoming param
    }	
}

proc act_GotDest { } {
    global dest
    
    set status [infotag get evt_status]
    
    if {  $status == "cd_004" } {
	set dest [infotag get evt_dcdigits]
	leg proceeding leg_incoming
	leg setup $dest callInfo leg_incoming
	#playtone leg_incoming tn_ringback
	
    } else {
	puts "\nCall [infotag get con_all] got event $status collecting destination"
	call close
    }
}
	

proc act_CallSetupDone { } {
    global beep
    global method

    set status [infotag get evt_status]

    set method "h323"
    set av-send(h323-ivr-out,10) "remote_ip:true"
    aaa accounting start leg_outgoing -a av-send -s $method

    if { $status == "ls_000"} {
	
	set  creditTimeLeft [infotag get leg_settlement_time leg_all]

	if { ($creditTimeLeft == "unlimited") || 
	     ($creditTimeLeft == "uninitialized") } {
	    puts "\n Unlimited Time"
	} else {
	    # start the timer for ...
	    if { $creditTimeLeft < 10 } {
		set beep 1
		set delay $creditTimeLeft
	    } else {
		set delay [expr $creditTimeLeft - 10]
	    }
	    timer start leg_timer $delay leg_incoming
	}
    } else {
	puts "Call [infotag get con_all] got event $status collecting destination"
	call close
    }
}

proc act_Timer { } {
    global beep 
    global incoming
    global outgoing

    set incoming [infotag get leg_incoming]
    set outgoing [infotag get leg_outgoing]
    
    if { $beep == 0 } {
	#insert a beep ...to the caller
	connection destroy con_all
	set beep 1
    } else {
        connection destroy con_all
        fsm setstate LASTWARN
    }
}

proc act_LastWarn { } {
    media play leg_incoming flash:out_of_time.au
}

proc act_Destroy { } {
    media play leg_incoming flash:beep.au 
}

proc act_Beeped { } {
    global incoming
    global outgoing

    connection create $incoming $outgoing
}

proc act_ConnectedAgain { } {
    timer start leg_timer 10 leg_incoming
}

proc act_Ignore { } {
# Dummy
    puts "Event Capture"
}

proc act_Cleanup { } {
    call close
}

init 

#----------------------------------
#   State Machine
#----------------------------------
  set fsm(any_state,ev_disconnected)  "act_Cleanup              same_state"
  
  set fsm(CALL_INIT,ev_setup_indication) "act_Setup            AUTHENTICATE"

  set fsm(AUTHENTICATE,ev_authorize_done) "act_Authenticated GETDEST"

  set fsm(GETDEST,ev_collectdigits_done)  "act_GotDest          PLACECALL"

  set fsm(PLACECALL,ev_setup_done)    "act_CallSetupDone    CALLACTIVE"


  set fsm(CALLACTIVE,ev_leg_timer)    "act_Timer            INSERTBEEP"
  set fsm(INSERTBEEP,ev_destroy_done) "act_Destroy          same_state"
  set fsm(INSERTBEEP,ev_media_done)   "act_Beeped           same_state"
  set fsm(INSERTBEEP,ev_create_done)    "act_ConnectedAgain   CALLACTIVE"

  set fsm(LASTWARN,ev_destroy_done)     "act_LastWarn        CALLDISCONNECT"
  
  set fsm(CALLACTIVE,ev_disconnected)   "act_Cleanup         CALLDISCONNECT"
  set fsm(CALLDISCONNECT,ev_disconnected)   "act_Cleanup         same_state"
  set fsm(CALLDISCONNECT,ev_media_done)     "act_Cleanup         same_state"
  set fsm(CALLDISCONNECT,ev_disconnect_done) "act_Cleanup         same_state"
  set fsm(CALLDISCONNECT,ev_leg_timer)      "act_Cleanup         same_state"

  fsm define fsm CALL_INIT




