<?
require "/usr/lib/cherry/monitoring/common.pl";
require "/usr/lib/cherry/dateutils";

use GD;
use POSIX qw(floor ceil);
use Time::Local qw(timelocal);

my $current_time = time;

($tree, $last_message) = common::restore_tree();

$from = date2unixtime($from_date) || $current_time;
$till = date2unixtime($till_date) || $current_time;

$scale = 'day' unless $scale eq 'year' or $scale eq 'month' or $scale eq 'week' or $scale eq 'day';

#
# output graphs
#

my(@selected, @visible);

common::traverse_tree($tree, \@selected, \@visible);

@selected = sort { common::smart_sort($a->[0], $b->[0]) } @selected;

my $count = scalar @selected;

unless($count) {
	?><SCRIPT>alert("$D_MONITORING_NO_ELEMENTS_SPECIFIED"); history.back(1);</SCRIPT><?
	return;
}

my $max_count = scalar @config::graphs_colors;

if($count > $max_count) {
	$message = sprintf($D_MONITORING_MAX_ELEMENTS_EXCEEDED, $max_count);
	?><SCRIPT>alert("$message"); history.back(1);</SCRIPT><?
	return;
}


?>
<BODY>
<TABLE class="page_h" > 
<TR><TD>$D_MONITORING_GRAPHS_TITLE</TD></TR>
</TABLE>
<FORM action="template.cgi?tpl=monitoring/view.tpl" method=post>
<INPUT type=submit value="$D_MONITORING_STATE"> <INPUT type=submit name=log value="$D_MONITORING_LOG"><BR>
<?
$index = 0;

foreach(map {$_->[0]} @selected) {
	?><INPUT type=hidden name="selected:$_" value=1><?
	$index++;
}

foreach(@visible) {
	?><INPUT type=hidden name="visible:$_" value=1><?
}

?>
</FORM>
<BR>
<?

#
# draws grid, labels and axis
#

sub draw_grid {
	my($im, $min, $max, $grid_scale, $min_steps, $grid_min_color, $grid_maj_color, $axis_color, $label_color, @maj_steps) = @_;

	my $grid_x_maj_step = $config::width / @maj_steps;
	my $grid_x_min_step = $grid_x_maj_step / $min_steps;

	for(my $x = 0; $x < $config::width; $x += $grid_x_maj_step) {
		$im->line($x + $config::left_margin, 0 + $config::top_margin, $x + $config::left_margin, $config::height - 1 + $config::top_margin, $grid_maj_color);

		my $begin = $grid_x_min_step + $x;
		my $end = $x + $grid_x_maj_step;

		my $center = ($x + $end) / 2;

		my $label = shift @maj_steps;

		my $label_length = $config::font_size[0] * length $label;

		my $label_x = $center - $label_length / 2;
		my $label_y = $config::height + 1;

		$im->string($config::font, $label_x + $config::left_margin, $label_y + $config::top_margin, $label, $label_color);

		for(my $x = $begin; $x < $end; $x += $grid_x_min_step) {
			$im->line($x + $config::left_margin, $config::top_margin, $x + $config::left_margin, $config::height - 1 + $config::top_margin, $grid_min_color);
		}
	}

	my $grid_y_maj_step = $config::height / ($max - $min);
	my $grid_y_min_step = $grid_y_maj_step / 5;
	
	my $center = $config::height + $grid_y_maj_step * $min;

	my $n = 0;

	for(my $y = $center; $y >= 0; $y -= $grid_y_maj_step) {
		my $label = $n * $grid_scale;
		$im->string($config::font, $config::left_margin - length($label) * $config::font_size[0] - 2, $y - $config::font_size[1] / 2 + $config::top_margin, $label, $label_color);

		$im->line($config::left_margin, $y + $config::top_margin, $config::left_margin + $config::width - 1, $y + $config::top_margin, $grid_maj_color);

		my $begin = $y - $grid_y_min_step;
		my $end = $y - $grid_y_maj_step;
		$end = 0 if $end < 0;

		for(my $y = $begin; $y > $end; $y -= $grid_y_min_step) {
			$im->line($config::left_margin, $y + $config::top_margin, $config::left_margin + $config::width - 1, $y + $config::top_margin, $grid_min_color);
		}

		$n++;
	}

	$n = 0;

	for(my $y = $center; $y < $config::height; $y += $grid_y_maj_step) {
		my $label = $n * $grid_scale;
		$im->string($config::font, $config::left_margin - length($label) * $config::font_size[0] - 2, $y - $config::font_size[1] / 2 + $config::top_margin, $label, $label_color);
		$im->line($config::left_margin, $y + $config::top_margin, $config::left_margin + $config::width - 1, $y + $config::top_margin, $grid_maj_color);

		my $begin = $y + $grid_y_min_step;
		my $end = $y + $grid_y_maj_step;

		$end = $config::height - 1 if $end > $config::height - 1;

		for(my $y = $begin; $y < $end; $y += $grid_y_min_step) {
			$im->line($config::left_margin, $y + $config::top_margin, $config::left_margin + $config::width - 1, $y + $config::top_margin, $grid_min_color);
		}
		$n--;
	}

	$im->line($config::left_margin, $center + $config::top_margin, $config::left_margin + $config::width - 1, $center + $config::top_margin, $axis_color);

}

#
# finds optimal scale factor
#

sub calc_scale {
	my($min, $max, $maj_y_steps) = @_;
	my $range = ($max - $min) / $maj_y_steps;

	return (1, 0, 1) unless $range;

	my $power = floor(log($range) / log(10));

	my $rest = $range / 10 ** $power;

	my $fraction = 1;

	if($rest >= 5) {
		$fraction = 10;
	} elsif($rest >= 2) {
		$fraction = 5;
	} elsif($rest >= 1) {
		$fraction = 2;
	}

	my $scale_factor = $fraction * 10 ** $power;

	die "line 172" unless $scale_factor;

	my $max_grid = $max / $scale_factor;
	my $min_grid = $min / $scale_factor;

	return ($scale_factor, $min_grid, $max_grid);
}

#
# draws data
#

sub draw_data {
	my($im, $shift, $color, $scale, $min_grid, $max_grid, $data, $offset, $slots) = @_;

	my $center = defined $scale ? $config::height + $min_grid * $config::height / ($max_grid - $min_grid) : $config::height - $max_count + $shift * $config::graphs_step;
	my $scale_factor = defined $scale ? $config::height / ($max_grid - $min_grid) / $scale : $config::height - $max_count;
	my $step = $config::width / $slots;

	my $data_length = scalar @{$data};

	$slots = $data_length - $offset if $slots > $data_length - $offset;

	my($min, $max, $summ, $count);

	$min = $max = $summ = $data->[$offset];
	$count = 1 if defined $summ;

	for(my $i = 1; $i < $slots; $i++) {

		my $slot_value = $data->[$offset + $i];

		if(defined $slot_value) {
			$min = $slot_value if $slot_value < $min;
			$max = $slot_value if $slot_value > $max;
			$summ += $slot_value;
			$count++;
		}

		unless(defined $scale) {
			if(defined $data->[$offset + $i - 1]) {
				my $y0 = $data->[$offset + $i - 1] * $scale_factor;
				my $x1 = $step * $i;
				my $x0 = $x1 - $step;

				$im->line($config::left_margin + $x0, $center - $y0 + $config::top_margin, $config::left_margin + $x1, $center - $y0 + $config::top_margin, $color);

				if(defined $slot_value) {
					my $y1 = $slot_value * $scale_factor;
					$im->line($config::left_margin + $x1, $center - $y0 + $config::top_margin, $config::left_margin + $x1, $center - $y1 + $config::top_margin, $color);
				}
			}
		} else {
			if(defined $data->[$offset + $i - 1] and defined $slot_value) {
				my $x1 = $step * $i;
				my $y1 = $slot_value * $scale_factor;
				my $x0 = $x1 - $step;
				my $y0 = $data->[$offset + $i - 1] * $scale_factor;
				$im->line($config::left_margin + $x0, $center - $y0 + $config::top_margin, $config::left_margin + $x1, $center - $y1 + $config::top_margin, $color);
			}
		}
	}

	return ($min, $max, $summ / $count, $summ, $count) if $count;
}

#
# draws piece of graphs
#

sub draw_tile {
	my($min_grid, $max_grid, $grid_scale, $offset, $slots, $maj_x_steps, $min_x_steps, %data) = @_;
	my $image = new GD::Image($config::width + $config::left_margin, $config::height + $config::font_size[1] + 1 + $config::top_margin);
	my $bg_color =  $image->colorAllocate(@config::bg_color);
	my $axis_color = $image->colorAllocate(@config::axis_color);
	my $label_color = $image->colorAllocate(@config::label_color);
	my $grid_min_color = $image->colorAllocate(@config::grid_min_color);
	my $grid_maj_color = $image->colorAllocate(@config::grid_maj_color);

	$image->transparent($bg_color);

	draw_grid($image, $min_grid, $max_grid, $grid_scale, $min_x_steps, $grid_min_color, $grid_maj_color, $axis_color, $label_color, @$maj_x_steps);
	my $shift = 0;

	my %summary;

	foreach(keys %data) {
		my $color = $image->colorAllocate(@{$data{$_}->[2]});
		my($min, $max, $average, $summ, $count) = draw_data($image, $shift, $color, $data{$_}->[4], $min_grid, $max_grid, $data{$_}->[3], $offset, $slots);
		$summary{$_} = [$min, $max, $average, $summ, $count] if $count;
		$shift++;
	}

	return ($image, %summary);
}

#
# outputs one tile
#

sub make_tile {
	my($min_grid, $max_grid, $grid_scale, $offset, $slots, $maj_x_steps, $min_x_steps, $from, $span, %data) = @_;

	my($image, %summary) = draw_tile($min_grid, $max_grid, $grid_scale, $offset, $slots, $maj_x_steps, $min_x_steps, %data);

	$name = time . int(rand(100000));

	open FH, ">$config::tmp_prefix$name" or die $!;
	print FH $image->png;
	close FH;

	return [$name, $from, $from + $span - 1, %summary];
}

sub process_value {
	my($from_time, $interval, $data, $accumulator, $timestamp, $slot, $path, $curr_value) = @_;

	$accumulator = $accumulator->{$path};

	my $value = $accumulator->{prev_value};

	my $data_type = $accumulator->{type};
	my $type_type = $common::data_types->{$data_type}->{type};

	my $compound = $common::types->{$type_type}->{compound} || \&common::average;

	if($process_value = $common::types->{$type_type}->{process_graph_value}) {
		$value = $process_value->($data_type, $accumulator->{prev_value}, $accumulator->{prev_time}, $curr_value, $timestamp);
	}

	my $prev_slot = defined $accumulator->{prev_time} ? floor(($accumulator->{prev_time} - $from_time) / $interval) : undef;

	if($slot == $prev_slot) {
		my $amount = ($timestamp - $accumulator->{prev_time}) / $interval;
		$accumulator->{compound_value} = defined $value && defined $accumulator->{compound_value} ? $compound->($accumulator->{compound_value}, $value, $amount) : undef
	} else {
		# timestamp at start of slot
		my $slot_time = $slot * $interval + $from_time;
		my $next_slot_time = ($prev_slot + 1) * $interval + $from_time;

		# fill previous slot, if it exists
		if(defined $prev_slot and $prev_slot >= 0) {
			my $amount = ($next_slot_time - $accumulator->{prev_time}) / $interval;

			$accumulator->{compound_value} =  defined $value && defined $accumulator->{compound_value} ? $compound->($accumulator->{compound_value}, $value, $amount) : undef;
			$data->{$path}->[3]->[$prev_slot] = $accumulator->{compound_value};

			my $value =  $accumulator->{compound_value};

			$data->{$path}->[0] = $value if $value < $data->{$path}->[0];
			$data->{$path}->[1] = $value if $value > $data->{$path}->[1];
		}

		# fill intermediate slots (if we have ones)
		$prev_slot = -1 if $prev_slot < 0;

		foreach((defined $prev_slot ? $prev_slot + 1 : 0)..($slot - 1)) {
			$data->{$path}->[3]->[$_] = $value;
		}

		if($slot - $prev_slot > 1) {
			$data->{$path}->[0] = $value if $value < $data->{$path}->[0];
			$data->{$path}->[1] = $value if $value > $data->{$path}->[1];
		}

		# start current slot
		my $amount = ($timestamp - $slot_time) / $interval;
		$accumulator->{compound_value} = defined $value ? $compound->(undef, $value, $amount) : undef;
	}

	$accumulator->{prev_time} = $timestamp;
	$accumulator->{prev_value} = $curr_value;
}

#
# finds appropriate prefix
#

sub find_prefix {
	my $scale = shift;
	my $order = floor(log($scale) / log(10)); 
	my $prefix = 0;
	my $dist = abs($prefix - $order);

	foreach(keys %$common::prefixen) {
		my $curr_dist = abs($_ - $order);
		if($curr_dist < $dist) {
			$prefix = $_;
			$dist = $curr_dist;
		}
	}

	my $factor = $scale / 10 ** $prefix;
	$prefix = $common::prefixen->{$prefix};

	return ($factor, $prefix);
}

#
# acquires data from database
#

sub get_data {
	my($data, $from_time, $till_time, $interval, $selected) = @_;

	my %accumulator;
	my $prev_slot;

	foreach my $element (@$selected) {
		my($path, $type, $color) = @$element;
		$data->{$path} = [] unless $data->{$path};
		my @color = map { hex($_) } $color =~ m/^\#(\w\w)(\w\w)(\w\w)/;
		$data->{$path}->[2] = \@color;
		$data->{$path}->[3] = [];

		$accumulator{$path} = {};

		$accumulator{$path}->{type} = $type;

		($accumulator{$path}->{prev_time}, $accumulator{$path}->{prev_value}) = $common::dbh->selectrow_array("SELECT timestamp, value FROM monitor_messages WHERE path = ? AND timestamp < ? ORDER BY timestamp DESC LIMIT 1", undef, $path, $from_time);
	}

	my $paths = join(', ', map { "?" } @$selected);

	my $sth = $common::dbh->prepare("SELECT timestamp, path, value FROM monitor_messages WHERE timestamp >= ? AND timestamp <= ? AND path IN ($paths) ORDER BY timestamp ASC");
	$sth->execute($from_time, $till_time, map { $_->[0] } @$selected);

	while(my($timestamp, $path, $curr_value) = $sth->fetchrow_array) {
		my $slot = floor(($timestamp - $from_time) / $interval);

		process_value($from_time, $interval, $data, \%accumulator, $timestamp, $slot, $path, $curr_value);
	}

	$till_time = $last_message if $till_time > $last_message;

	my $slot = floor(($till_time - $from_time) / $interval);

	foreach my $element (@$selected) {
		my $path = $element->[0];
		my($timestamp, $value) = $common::dbh->selectrow_array("SELECT timestamp, value FROM monitor_messages WHERE path = ? AND timestamp > ? ORDER BY timestamp ASC LIMIT 1", undef, $path, $till_time);
		$timestamp = $till_time unless $timestamp;
		process_value($from_time, $interval, $data, \%accumulator, $timestamp, $slot, $path, $value);
	}
}

#
# draws whole graphs
#

sub draw_graphs {
	my($maj_y_steps, $scale, $from_time, $till_time, $tiles, $selected) = @_;

	my($interval, $period, $length);
	my($min_x_steps, $y_steps);

	my(@maj_x_steps, $slots);

	if($scale eq 'day') {
		$interval = 300;
		$period = 24 * 3600;
		($min_x_steps, @maj_x_steps) = (2, map { sprintf("%02d", $_) } 0..23);
		$slots = $period / $interval;

		my($from_day, $from_year) = (localtime($from_time))[7, 5];
		my($till_day, $till_year) = (localtime($till_time))[7, 5];
		$from_time = timelocal(0, 0, 0, 1, 0, $from_year) + $from_day * 3600 * 24;
		$till_time = timelocal(59, 59, 23, 1, 0, $till_year) + $till_day * 3600 * 24;

	}

	if($scale eq 'week') {
		$interval = 3600;
		$period = 24 * 3600 * 7;
		($min_x_steps, $y_steps, @maj_x_steps) = (8, 4, qw(      ));
		$slots = $period / $interval;
		my($from_day, $from_wday, $from_year) = (localtime($from_time))[7, 6, 5];
		my($till_day, $till_wday, $till_year) = (localtime($till_time))[7, 6, 5];

		$from_time = timelocal(0, 0, 0, 1, 0, $from_year) + ($from_day - ($from_wday ? $from_wday - 1 : 6)) * 3600 * 24;
		$till_time = timelocal(59, 59, 23, 1, 0, $till_year) + ($till_day + ($till_wday ? (7 - $till_wday) : 0)) * 3600 * 24;
	}

	if($scale eq 'month') {
		$interval = 4 * 3600;
		$min_x_steps = 2;
		my($from_mon, $from_year) = (localtime($from_time))[4, 5];
		my($till_mon, $till_year) = (localtime($till_time))[4, 5];
		$from_time = timelocal(0, 0, 0, 1, $from_mon, $from_year);
		$till_time = timelocal(0, 0, 0, 1, ($till_mon + 1) % 12, int(($till_mon + 1) / 12) + $till_year) - 1;

		$slots = [];

		for(my $i = $from_mon + 12 * $from_year; $i <= $till_mon + 12 * $till_year; $i++)  {
			my $mon = $i % 12;
			my $year = int($i / 12);
	 		my $next_mon = ($i + 1) % 12;
			my $next_year = int(($i + 1) / 12);

			my $days = ceil((timelocal(0, 0, 0, 1, $next_mon, $next_year) - timelocal(0, 0, 0, 1, $mon, $year)) / (3600 * 24));
			push @maj_x_steps, [1..$days];
			push @$slots, 3600 * 24 * $days / $interval;
		}
	}

	if($scale eq 'year') {
		$interval = 4 * 3600 * 12;
		$min_x_steps = 4;
		my $from_year = (localtime($from_time))[5];
		my $till_year = (localtime($till_time))[5];
		$from_time = timelocal(0, 0, 0, 1, 0, $from_year);
		$till_time = timelocal(0, 0, 0, 1, 0, $till_year + 1) - 1;

		$slots = [];

		for(my $i = $from_year; $i <= $till_year; $i++) {
			my $period = (timelocal(0, 0, 0, 1, 0, $i + 1) - timelocal(0, 0, 0, 1, 0, $i));
			push @$slots, $period / $interval;
			push @maj_x_steps, [qw(           )];
		}
	}

	$length = ($till_time - $from_time + 1) / $interval;

	my($min_grid, $max_grid) = (0, 0);

	# get data here
	my %data; # [$min, $max, $color, [@data], $scale]

	get_data(\%data, $from_time, $till_time, $interval, $selected);

	my %types;

	my $base_type = undef;

	foreach(@$selected) {
		my($path, $type) = @$_;

		$data{$path}->[5] = $type;

		$types{$type} = [0, 0] unless $types{$type};

		unless($common::types->{$common::data_types->{$type}->{type}}->{not_scalable}) {
			my($min, $max) = @{ $data{$path} };

			$types{$type}->[0] = $min if $min < $types{$type}->[0];
			$types{$type}->[1] = $max if $max > $types{$type}->[1];

			$base_type = $type if $path eq $base;
		} else {
			$types{$type}->[5] = 1;
		}
	}

	my $grid_scale = 1;

	foreach(keys %types) {
		my($min, $max, undef, undef, undef, $not_scalable) = @{ $types{$_} };
		unless($not_scalable) {
			my($scale, $loc_min_grid, $loc_max_grid) = calc_scale($min, $max, $config::maj_y_steps);
			$min_grid = $loc_min_grid if $loc_min_grid < $min_grid;
			$max_grid = $loc_max_grid if $loc_max_grid > $max_grid;

			($grid_scale) = find_prefix($scale) if $_ eq $base_type;

			$types{$_}->[2] = $scale;
		} else {
			$types{$_}->[2] = undef;
		}
	}

	foreach(values %types) {
		unless($_->[5]) {
			my($factor, $prefix) = find_prefix($_->[2] / $grid_scale);
			$_->[3] = $factor;
			$_->[4] = $prefix;
		}
	}

	foreach(keys %data) {
		my $type = $data{$_}->[5];
		my $scale = $types{$type}->[2];

		$data{$_}->[4] = $scale;
	}

	foreach(@$selected) {
		my $path = $_->[0];
		my $type = $data{$path}->[5];
		$_->[3] = $types{$type}->[3];
		$_->[4] = $types{$type}->[4];
		$_->[5] = $types{$type}->[5];
		$_->[6] = $types{$type}->[2] / $grid_scale;
	}

=begin
	use Data::Dumper;
	print "<PRE>";
	print Dumper(\%types);
	print "</PRE>";
	exit;
=cut

	if(ref($slots) eq 'ARRAY') {
		my $offset = 0;
		while($offset < $length) {
			my $tile_slots = shift @$slots;
			my $maj_x_steps = shift @maj_x_steps;
			push @$tiles, make_tile($min_grid, $max_grid, $grid_scale, $offset, $tile_slots, $maj_x_steps, $min_x_steps, $from_time + $offset * $interval, $tile_slots * $interval, %data);
			$offset += $tile_slots;
		}
	} else {
		for(my $offset = 0; $offset < $length; $offset += $slots) {
			push @$tiles, make_tile($min_grid, $max_grid, $grid_scale, $offset, $slots, \@maj_x_steps, $min_x_steps, $from_time + $offset * $interval, $slots * $interval, %data);
		}
	}
}

#
# here we go
#

my $label = 0;

foreach(@selected) {
	($path, $type) = @$_;
	$color = $config::graphs_colors[$label];
	$_->[2] = $color;
	$label++;
}

?><BR><?

my @tiles;

draw_graphs(5, $scale, $from, $till, \@tiles, \@selected);

my %summary;

foreach(@tiles) {
	my(undef, undef, undef, %tile_summary) = @$_;

	foreach(keys %tile_summary) {
		if($tile_summary{$_}) {
			$summary{$_} = [] unless $summary{$_};
			my($min, $max, undef, $summ, $count) = @{ $tile_summary{$_} };
			$summary{$_}->[0] = $min if $min < $summary{$_}->[0];
			$summary{$_}->[1] = $max if $max > $summary{$_}->[1];

			$summary{$_}->[2] += $summ;
			$summary{$_}->[3] += $count;
		}
	}
}

foreach(@selected) {
	my($path, $type, $color, $factor, $prefix, undef, $scale) = @$_;

	my $units = $common::data_types->{$type}->{graph_units} || $common::data_types->{$type}->{units} || 'units';

	if($factor) {
		my($min, $max, $summ, $count) = @{ $summary{$path} };

		my $average = $summ / $count if $count;

		$min = sprintf("%.2f", $min / $scale);
		$max = sprintf("%.2f", $max / $scale);
		$average = sprintf("%.2f", $average / $scale);

		my $summary = ". $D_MONITORING_MINIMUM: $min, $D_MONITORING_MAXIMUM: $max, $D_MONITORING_AVERAGE: $average";

		?><DIV style="color: $color">$path: $type. ($factor $prefix$units$summary)</DIV><?
	} else {
		?><DIV style="color: $color">$path: $type. ($units)</DIV><?
	}
}

?><TABLE border=0 cellpadding=0 cellspacing=2><?

foreach(@tiles) {
	my($id, $from, $till, %summary) = @$_;
	$from = unixtime2date($from, $D_TIME_FORMAT);
	$till = unixtime2date($till, $D_TIME_FORMAT);
	?><TR><TD align=center><BR>$from - $till</TD></TR><?
	?><TR><TD align=center><IMG src="image.cgi?tpl=monitoring/graph.img&pic=$id"></TD></TR><?
	if(%summary) {
		foreach(@selected) {
			my($path, $type, $color, $factor, $prefix, undef, $scale) = @$_;

			if($summary{$path} and $scale) {
				my($min, $max, $average) = @{ $summary{$path} };

				my $units = $common::data_types->{$type}->{graph_units} || $common::data_types->{$type}->{units} || 'units';

				$min = sprintf("%.2f", $min / $scale);
				$max = sprintf("%.2f", $max / $scale);
				$average = sprintf("%.2f", $average / $scale);
			
				?><TR><TD style="color: $color" align=center>$D_MONITORING_MINIMUM: $min $prefix$units, $D_MONITORING_MAXIMUM: $max $prefix$units, $D_MONITORING_AVERAGE: $average $prefix$units</TD></TR><?
			}
		}
	}
}

?>
</TABLE>
</BODY>
</HTML>
