<?
require "/usr/lib/cherry/monitoring/common.pl";
require "/usr/lib/cherry/dateutils";

my $current_time = time;

$tree = common::restore_tree();

$from = date2unixtime($from_date);
$till = date2unixtime($till_date, '23:59:59') || $current_time;

#
# print data row
#

sub print_row {
	my($time, $values, $summary) = @_;

	$time = unixtime2date($time, $D_TIME_FORMAT);

	print <<EOF;
<TR>
<TD width=90>$time</TD>
EOF

	if(ref $values eq 'ARRAY') {
		my $index = 0;
		foreach my $node (@$values) {
			my $value = $D_MONITORING_UNDEFINED_VALUE;

			$color = defined $node->{e_level} ? $config::e_colors[$node->{e_level} + 1] : $config::e_colors[0];

			if($process_value = $common::types->{$node->{type}}->{process_value}) {
				$value = $process_value->($node->{prev_value}, $node->{prev_timestamp}, $node->{value}, $node->{timestamp}, $summary->[$index]);
			} else {
				$value = $node->{value};
			}
			$value = $D_MONITORING_UNDEFINED_VALUE unless defined $value;
			?><TD align=right style="color: $color"> $value </TD><?
			$index++;
		}
	} else {
		?><TD colspan="$values" align=center> $D_MONITORING_SERVER_DEAD </TD><?
	}

?></TR>
<?
}

#
# output log
#

my(@selected, @visible);

common::traverse_tree($tree, \@selected, \@visible);

@selected = sort { common::smart_sort($a->[0], $b->[0]) } @selected;

my $count = scalar @selected;

unless($count) {
	?><SCRIPT>alert("$D_MONITORING_NO_ELEMENTS_SPECIFIED"); history.back(1);</SCRIPT><?
	return;
}

?>
<BODY>
<TABLE class="page_h" > 
<TR><TD> </TD></TR>
</TABLE>
<FORM action="template.cgi?tpl=monitoring/view.tpl" method=post>
<INPUT type=hidden name=from_date value="$from_date">
<INPUT type=hidden name=till_date value="$till_date">
<INPUT type=submit value="$D_MONITORING_STATE"> <INPUT type=submit name=graphs value="$D_MONITORING_GRAPHS">
<SELECT name=scale>
<OPTION value=day>$D_MONITORING_SCALE_DAY</OPTION>
<OPTION value=week>$D_MONITORING_SCALE_WEEK</OPTION
<OPTION value=month>$D_MONITORING_SCALE_MONTH</OPTION>
<OPTION value=year>$D_MONITORING_SCALE_YEAR</OPTION>
</SELECT>
<BR>
<?
foreach(map {$_->[0]} @selected) {
	?><INPUT type=hidden name="selected:$_" value=1><?
}

foreach(@visible) {
	?><INPUT type=hidden name="visible:$_" value=1><?
}

?>
</FORM>
<BR>
<?
my $label = 0;

foreach(@selected) {
	my $symlabel = chr(ord('A') + $label);
	($path, $type) = @$_;
	?><B>$symlabel:</B> $path: $type<BR><?
	$label++;
}

?>
<BR>
<TABLE border=1 width=99%>
<THEAD>
<TH></TH>
<?

for(my $index = 0; $index < $count; $index++) {
	my $path = $selected[$index]->[0];
	$label = chr(ord('A') + $index);
	$units = $common::data_types->{$selected[$index]->[1]}->{units};
	?><TH>$label ($units)</TH><?
}

?></THEAD><?

@clauses = ();

if($from) {
	push @clauses, "timestamp > $from";
}

if($till) {
	push @clauses, "timestamp < $till";
}

$clause = @clauses ? 'AND ' . join(' AND ', @clauses) . ' ' : '';

my $query = qq|SELECT timestamp, path, value, e_level FROM monitor_messages WHERE (path IN (| . join(', ', map("?", @selected) ) . qq|) OR path IS NULL) ${clause}ORDER BY timestamp ASC|;

my $sth = $common::dbh->prepare($query);
$sth->execute(map {$_->[0]} @selected);

my $last_time = 0;
my $last_alive = 0;
my $last_was_no_data = 1;

my @values;
my @summary;

for(my $index = 0; $index < $count; $index++) {

	$values[$index]->{type} = $common::data_types->{$selected[$index]->[1]}->{type};
	$values[$index]->{prev_value} = undef;
	$values[$index]->{value} = undef;
	$values[$index]->{prev_timestamp} = undef;
	$values[$index]->{timestamp} = undef;

	$summary[$index] = {};
}

while(my($timestamp, $path, $value, $e_level) = $sth->fetchrow_array) {
	if($last_alive + $config::server_alive_period + $config::server_alive_period_treshold < $timestamp) {
		print_row($last_alive, scalar @selected) unless $last_was_no_data;
		$last_was_no_data = 1;
		$last_time = $timestamp;
	}

	$last_alive = $timestamp;

	if(defined $path) {
		$index = common::find_index($path, \@selected);

		$c_value = $values[$index]->{value};
		if($timestamp > $last_time + 60 or defined $e_level and defined $c_value and $c_value ne $value) {
			if($last_time) {
				print_row($last_time, \@values, \@summary);
				$last_was_no_data = undef;

				foreach(@values) {
					$_->{prev_value} = $_->{value};
					$_->{prev_timestamp} = $_->{timestamp};
				}
			}

			$last_time = $timestamp;
		}

		$values[$index]->{value} = $value;
		$values[$index]->{timestamp} = $timestamp;
		$values[$index]->{e_level} = $e_level;
	}
}

if($last_alive) {
	if($last_alive < $till - $config::server_alive_period - $config::server_alive_period_treshold) {
		print_row($last_alive, scalar @selected);
	} else {
		print_row($till, \@values, \@summary);
	}
}

my %footer;

my $index = 0;
foreach(@summary) {
	if($process_summary = $common::types->{$values[$index]->{type}}->{process_summary}) {
		foreach( $process_summary->($_)) {
			my($label, $value) = @$_;
			$footer{$label}->[$index] = $value;
		}
	}

	$index++;
}

foreach my $label (sort keys %footer) {
	@values = @{$footer{$label}};
	print "<TR><TD>$label:</TD>";
	foreach(@values) {
		print "<TD align=right>$_</TD>";
	}
	print '</TR>';
}

?></TABLE>
</BODY>
</HTML>
